#unittest.skip np.random not supported yet
import numpy as np
def step(dt, prices, c0, c1, noises):
    return prices * np.exp(c0 * dt + c1 * noises)

def monte_carlo_pricer(paths, dt, interest, volatility):
    c0 = interest - 0.5 * volatility ** 2
    c1 = volatility * np.sqrt(dt)

    for j in range(1, paths.shape[1]): # for all trials
        prices = paths[:, j - 1]
        # generate normally distributed random number
        noises = np.random.normal(0., 1., prices.size)
        # calculate the next batch of prices for all trials
        paths[:, j] = step(dt, prices, c0, c1, noises)



""" Check the random module behavior. """
from pythran.tests import TestEnv


@TestEnv.module
class TestRandom(TestEnv):

    """
    Check implementation for usual random functions.

    Functions are:
    - random.random
    - random.gauss
    - random.uniform
    - random.expovariate
    - random.randrange
    - random.randint
    - random.sample
    - random.choice
    - random.seed
    """

    def test_random(self):
        """ Check random.random is a uniform distributed generator. """
        self.run_test("""
        def random(n):
            from random import random
            s = sum(random() for x in range(n))
            return abs(s / n - 0.5) < .05""",
                      10 ** 5, random=[int])

    def test_gauss(self):
        """ Check random.gauss distribution. """
        self.run_test("""
        def gauss(n, mu, sigma):
            from random import gauss
            s = sum(gauss(mu,sigma) for x in range(n))
            return abs(s / n - mu) / sigma < .05""",
                      10 ** 6, 5, 2, gauss=[int, int, int])

    def test_uniform(self):
        """ Check uniform distribution between b and e. """
        self.run_test("""
        def uniform(n, b, e):
            from random import uniform
            s = sum(uniform(b, e) for x in range(n))
            return abs(s / n - (b + e) * .5) < .05""",
                      10 ** 6, 5, 25, uniform=[int, int, int])

    def test_expovariate(self):
        """ Check expovariate distribution. """
        self.run_test("""
        def expovariate(n, l):
            from random import expovariate
            s = sum(expovariate(l) for x in range(n))
            return abs(s / n - 1 / l) < .05""",
                      10 ** 6, 5., expovariate=[int, float])

    def test_randrange0(self):
        """ Check randrange with only end value. """
        self.run_test("""
        def randrange0(n):
            from random import randrange
            s = sum(randrange(n) for x in range(n))
            return abs(s / float(n * n) - .5) < .05""",
                      10 ** 5, randrange0=[int])

    def test_randrange1(self):
        """ Check randrange with begin and end. """
        self.run_test("""
        def randrange1(n):
            from random import randrange
            s = sum(randrange(-n, n + 1) for x in range(n))
            return abs(s / float(n * n)) < .05""", 10 ** 5, randrange1=[int])

    def test_randrange2(self):
        """ Check randrange step generated values. """
        self.run_test("""
        def randrange2(n):
            from random import randrange
            return all(randrange(3, n, 3) % 3 == 0 for x in range(n))""",
                      10 ** 4, randrange2=[int])

    def test_randint(self):
        """ Check randint distribution. """
        self.run_test("""
        def randint(n):
            from random import randint
            s = sum(randint(0,100) for x in range(n))
            return abs(s / float(n * 100) - 1 / 2.) < .05""",
                      10 ** 6, randint=[int])

    def test_sample_(self):
        """ Check sample picked values distribution. """
        self.run_test("""
        def sample(n, k):
            from random import sample
            s = sum(sum(sample(range(100), k)) for x in range(n))
            return abs(s / float(k * n) - 99 / 2.) < .05""",
                      10 ** 6, 4, sample=[int, int])

    def test_shuffle1(self):
        """ Check shuffling with default random function. """
        self.run_test("""
        def shuffle1(n):
            from random import shuffle
            r = list(range(n))
            shuffle(r)
            return r != list(range(n)) and sorted(r) == list(range(n))""",
                      10, shuffle1=[int])

    def test_shuffle2(self):
        """ Check shuffling with custom function. """
        self.run_test("""
        def shuffle2(n):
            from random import shuffle
            r = list(range(n))
            shuffle(r, lambda: 0)
            return r != list(range(n)) and sorted(r) == list(range(n))""",
                      10 ** 4, shuffle2=[int])

    def test_shuffle3(self):
        """ Check shuffling with random function. """
        self.run_test("""
        def shuffle3(n):
            from random import shuffle, random
            r = list(range(n))
            shuffle(r, random)
            return r != list(range(n)) and sorted(r) == list(range(n))""",
                      10 ** 4, shuffle3=[int])

    def test_choice(self):
        """ Check choice picked values distribution. """
        self.run_test("""
        def choice(n):
            from random import choice
            s = sum(choice(range(100)) for x in range(n))
            return abs(s / float(n) - 99 / 2.) < .05""",
                      10 ** 7, choice=[int])

    def test_random_seed(self):
        """ Check seeded random generate always the same value. """
        self.run_test("""
        def random_seed():
            from random import random, seed
            seed(1)
            a = random()
            seed(1)
            b = random()
            return a == b""", random_seed=[])



import omp
def omp_task_imp_firstprivate():
    i = 5
    k = 0
    result = False
    NUM_TASKS = 25
    task_result = True

    if 'omp parallel firstprivate(i)':
        in_parallel = omp.in_parallel()
        if 'omp single':
            for k in range(NUM_TASKS):
                if 'omp task  shared(result, task_result)':
                    if i != 5:
                        task_result = False

                    for j in range(0, NUM_TASKS):
                        i += 1
            'omp taskwait'
            result = task_result and i == 5
    return result or not in_parallel



#from https://bitbucket.org/FedericoV/numpy-tip-complex-modeling/src/806c968e3705/src/simulations/list_arc_distance.py?at=default
from math import sin, cos, atan2, sqrt, pi
from random import random

#pythran export arc_distance_list( (float, float) list, (float, float) list)
#runas arc_distance_list([(12.4,0.5),(-5.6,12.34),(9.21,-5.6),(3.4,2.3),(-23.31,12.6)],[(3.45,1.5),(55.4,567.0),(43.2,56.1),(3.4,1.34),(-56.9,-3.4)])
#bench import random; N=1000; a = [(random.random(), random.random()) for i in range(N)]; b = [(random.random(), random.random()) for i in range(N)]; arc_distance_list(a,b)
def arc_distance_list(a, b):
    distance_matrix = []
    for theta_1, phi_1 in a:
        temp_matrix = [ 2 * (atan2(sqrt(temp), sqrt(1 - temp))) for temp in [ sin((theta_2 - theta_1) / 2) ** 2 + cos(theta_1) * cos(theta_2) * sin((phi_2 - phi_1) / 2) ** 2  for theta_2, phi_2 in b ] ]
        distance_matrix.append(temp_matrix)

    return distance_matrix

#print(arc_distance_list([(12.4,0.5),(-5.6,12.34),(9.21,-5.6),(3.4,2.3),(-23.31,12.6)],[(3.45,1.5),(55.4,567.0),(43.2,56.1),(3.4,1.34),(-56.9,-3.4)]))



#pythran export deriv(int, float, complex, complex list, complex list, complex list, complex list list, float)
#runas deriv(3,4.5,complex(2,3),[complex(3,4),complex(1,2),complex(2,3),complex(5,6)],[complex(1,2),complex(2,3),complex(5,6),complex(3,4)],[complex(2,3),complex(3,4),complex(1,2),complex(5,6)],[[complex(2,3),complex(3,4),complex(1,2),complex(5,6)],[complex(2,3),complex(3,4),complex(1,2),complex(5,6)],[complex(2,3),complex(3,4),complex(1,2),complex(5,6)],[complex(2,3),complex(3,4),complex(1,2),complex(5,6)],[complex(2,3),complex(3,4),complex(1,2),complex(5,6)]],3.5)
def deriv(n,sig,alp,dg,dh1,dh3,bin,nu):
    dh2=[complex(0,0) for _ in dh1]
    ci = complex(0.0,1.0)
    dh1[0]=complex(0.5,0)*ci*complex(sig,0)
    exp1 = complex(-0.5,0)
    dh2[0]=alp;
    exp2 = complex(-1.0,0)
    dh3[0]=-2.0*nu
    exp3 = complex(-1.0,0)
    for i in range(1,n):
        dh1[i]=dh1[i-1]*exp1
        exp1=exp1-1.0
        dh2[i]=dh2[i-1]*exp2
        exp2=exp2-1.0
        dh3[i]=-nu*dh3[i-1]*exp3
        exp3=exp3-1.0
    dg[0]=complex(1.0)
    dg[1]=dh1[0]+dh2[0]+dh3[0]
    for i in range(2,n+1):
        dg[i]=dh1[i-1]+dh2[i-1]+dh3[i-1]
        for j in range(1,i):
            dg[i]=dg[i]+bin[j-1][i-1]*(dh1[j-1]+dh2[j-1]+dh3[j-1])*dg[i-j]
    return dg



import numpy as np


#import parameters as p
#pythran export tri(int [][],int [][])
#runas import numpy as np; a = np.arange(100, 0, -1).reshape(10, 10); tri(a, a)

def tri(score,score_t):
    score_t[0,0]=score[0,0]
    score_t=score[np.argsort(score[:,1])]
    return score_t



"""
HasReturn detects if there's a return or yield statement
HasBreak detects if there's a break statement
HasContinue detects if there's a continue statement
"""

from pythran.passmanager import NodeAnalysis


class HasReturn(NodeAnalysis):

    def __init__(self):
        self.result = False
        super(HasReturn, self).__init__()

    def visit_Return(self, _):
        self.result = True

    def visit_Yield(self, _):
        self.result = True


class HasBreak(NodeAnalysis):

    def __init__(self):
        self.result = False
        super(HasBreak, self).__init__()

    def visit_For(self, _):
        return

    visit_While = visit_For

    def visit_Break(self, _):
        self.result = True


class HasContinue(NodeAnalysis):

    def __init__(self):
        self.result = False
        super(HasContinue, self).__init__()

    def visit_For(self, _):
        return

    visit_While = visit_For

    def visit_Continue(self, _):
        self.result = True



#from http://rosettacode.org/wiki/Greatest_common_divisor#Python
#pythran export gcd_iter(int, int)
#pythran export gcd(int, int)
#pythran export gcd_bin(int, int)
#runas gcd_iter(40902, 24140)
#runas gcd(40902, 24140)
#runas gcd_bin(40902, 24140)

def gcd_iter(u, v):
    while v:
        u, v = v, u % v
    return abs(u)

def gcd(u, v):
    return gcd(v, u % v) if v else abs(u)

def gcd_bin(u, v):
    u, v = abs(u), abs(v) # u >= 0, v >= 0
    if u < v:
        u, v = v, u # u >= v >= 0
    if v == 0:
        return u

    # u >= v > 0
    k = 1
    while u & 1 == 0 and v & 1 == 0: # u, v - even
        u >>= 1; v >>= 1
        k <<= 1

    t = -v if u & 1 else u
    while t:
        while t & 1 == 0:
            t >>= 1
        if t > 0:
            u = t
        else:
            v = -t
        t = u - v
    return u * k



import unittest

unittest.skip
#segfault ....
def omp_parallel_num_threads():
    import omp
    max_threads = 0
    failed = 0

    if 'omp parallel':
        if 'omp master':
            max_threads = omp.get_num_threads()

    for threads in range(1, max_threads + 1):
        nthreads = 0
        if 'omp parallel reduction(+:failed) num_threads(threads)':
            failed += (threads != omp.get_num_threads())
            'omp atomic'
            nthreads += 1
        failed += (nthreads != threads)

    return not failed



import unittest
import pythran
import os.path

#pythran export a((float,(int,uintp),str list) list list)
#pythran export a(str)
#pythran export a( (str,str), int, intp list list)
#pythran export a( float set )
#pythran export a( bool:str dict )
#pythran export a( float )
#pythran export a( int8[] )
#pythran export a( int8[][]  order (F))
#pythran export a( byte )
#pythran export a0( uint8 )
#pythran export a1( int16 )
#pythran export a2( uint16 )
#pythran export a3( int32 )
#pythran export a4( uint32 )
#pythran export a5( int64 )
#pythran export a6( uint64 )
#pythran export a7( float32 )
#pythran export a8( float64 )
#pythran export a9( complex64 )
#pythran export a10( complex128 )
#pythran export a( int8 set )
#pythran export b( int8 set? )
#pythran export a( uint8 list)
#pythran export a( int16 [], slice)
#pythran export a( uint16 [][] order(C))
#pythran export a( uint16 [::][])
#pythran export a( uint16 [:,:,:])
#pythran export a( uint16 [:,::,:])
#pythran export a( uint16 [,,,,])
#pythran export a( (int32, ( uint32 , int64 ) ) )
#pythran export a( uint64:float32 dict )
#pythran export a( float64, complex64, complex128 )

class TestSpecParser(unittest.TestCase):

    def test_parser(self):
        real_path = os.path.splitext(os.path.realpath(__file__))[0]+".py"
        with open(real_path) as fd:
            print(pythran.spec_parser(fd.read()))

    def test_invalid_specs0(self):
        code = '#pythran export foo()\ndef foo(n): return n'
        with self.assertRaises(pythran.syntax.PythranSyntaxError):
            pythran.compile_pythrancode("dumber", code)

    def test_invalid_specs1(self):
        code = '#pythran export boo(int)\ndef boo(): pass'
        with self.assertRaises(pythran.syntax.PythranSyntaxError):
            pythran.compile_pythrancode("dumber", code)

    def test_invalid_specs2(self):
        code = '#pythran export bar(int)\ndef foo(): pass'
        with self.assertRaises(pythran.syntax.PythranSyntaxError):
            pythran.compile_pythrancode("dumber", code)

    def test_invalid_specs3(self):
        code = '#pythran export bar(int, int?, int)\ndef bar(x, y=1, z=1): pass'
        with self.assertRaises(pythran.syntax.PythranSyntaxError):
            pythran.compile_pythrancode("dumber", code)

    def test_multiline_spec0(self):
        code = '''
#pythran export foo(
              # )
def foo(): return
            '''
        self.assertTrue(pythran.spec_parser(code))

    def test_multiline_spec1(self):
        code = '''
#pythran export foo(int
  #, int
              # )
def foo(i,j): return
            '''
        self.assertTrue(pythran.spec_parser(code))

    def test_multiline_spec2(self):
        code = '''
#      pythran export foo(int,

# float
  #, int
              # )
def foo(i,j,k): return
            '''
        self.assertTrue(pythran.spec_parser(code))

    def test_crappy_spec0(self):
        code = '''
#      pythran export soo(int) this is an int test
def soo(i): return
            '''
        self.assertTrue(pythran.spec_parser(code))

    def test_crappy_spec1(self):
        code = '''
#      pythran export poo(int)
#this is a pythran export test
def poo(i): return
            '''
        self.assertTrue(pythran.spec_parser(code))

    def test_middle_spec0(self):
        code = '''
def too(i): return
#      pythran export too(int)
#this is a pythran export test
def bar(i): return
            '''
        self.assertTrue(pythran.spec_parser(code))


    def test_middle_spec1(self):
        code = '''
def zoo(i): return
#this is a pythran export test
#      pythran export zoo(int)
#this is an export test
#      pythran export zoo(str)
def bar(i): return
            '''
        self.assertEquals(len(pythran.spec_parser(code).functions), 1)
        self.assertEquals(len(pythran.spec_parser(code).functions['zoo']), 2)

    def test_var_export0(self):
        code = '''
#      pythran export coo
coo = 1
            '''
        self.assertTrue(pythran.spec_parser(code))



def omp_parallel_reduction():
    import math
    dt = 0.5
    rounding_error = 1.E-9

    sum = 0
    dsum = 0
    dt = 1. / 3.
    result = True
    product = 1
    logic_and = 1
    logic_or = 0
    bit_and = 1
    bit_or = 0
    exclusiv_bit_or = 0

    known_sum = (1000 * (1000 + 1)) / 2

    'omp parallel for schedule(dynamic,1) reduction(+:sum)'
    for i in range(1,1001):
        sum += i

    if known_sum != sum:
        print("E: reduction(+:sum)")
        result = False

    diff = (1000 * (1000 + 1)) / 2

    'omp parallel for schedule(dynamic,1) reduction(-:diff)'
    for i in range(1,1001):
        diff -= i

    if diff != 0:
        print("E: reduction(-:diff)")
        result = False

    dsum = 0
    dpt = 0
    for i in range(0, 20):
        dpt *= dt
    dknown_sum = (1 - dpt) / (1 - dt)

    'omp parallel for schedule(dynamic,1) reduction(+:dsum)'
    for i in range(0,20):
        dsum += math.pow(dt, i)

    if abs(dsum-dknown_sum) > rounding_error:
        print("E: reduction(+:dsum)")
        result = False

    dsum = 0
    dpt = 1
    for i in range(0, 20):
        dpt *= dt
    ddiff = (1 - dpt) / (1 - dt)

    'omp parallel for schedule(dynamic,1) reduction(-:ddiff)'
    for i in range(0,20):
        ddiff -= math.pow(dt, i)

    if abs(ddiff) > rounding_error:
        print("E: reduction(-:ddiff)")
        result = False

    'omp parallel for schedule(dynamic,1) reduction(*:product)'
    for i in range(1,11):
        product *= i

    known_product = 3628800

    if known_product != product:
        print("E: reduction(*:product)")
        result = False

    logics = [1 for i in range(0,1000)]

    'omp parallel for schedule(dynamic,1) reduction(&&:logic_and)'
    for i in range(0, 1000):
        logic_and = (logic_and and logics[i])

    if not logic_and:
        print("E: reduction(&&:logic_and)")
        result = False

    logic_and = 1;
    logics[1000//2]=0

    'omp parallel for schedule(dynamic,1) reduction(&&:logic_and)'
    for i in range(0, 1000):
        logic_and = (logic_and and logics[i])

    if logic_and:
        print("E: reduction(&&:logic_and) with logics[1000/2]=0")
        result = False

    logics = [0 for i in range(0,1000)]

    'omp parallel for schedule(dynamic,1) reduction(||:logic_or)'
    for i in range(0, 1000):
        logic_or = (logic_or or logics[i])

    if logic_or:
        print("E: reduction(||:logic_or)")
        result = False

    logic_or = 0;
    logics[1000//2]=1

    'omp parallel for schedule(dynamic,1) reduction(||:logic_or)'
    for i in range(0, 1000):
        logic_or = (logic_or or logics[i])

    if not logic_or:
        print("E: reduction(||:logic_or) with logics[1000/2]=1")
        result = False

    logics = [1 for i in range(0,1000)]

    'omp parallel for schedule(dynamic,1) reduction(&:bit_and)'
    for i in range(0, 1000):
        bit_and = (bit_and & logics[i])

    if not bit_and:
        print("E: reduction(&:bit_and)")
        result = False

    bit_and = 1;
    logics[1000//2]=0

    'omp parallel for schedule(dynamic,1) reduction(&:bit_and)'
    for i in range(0, 1000):
        bit_and = (bit_and & logics[i])

    if bit_and:
        print("E: reduction(&:bit_and) with logics[1000/2]=0")
        result = False

    logics = [0 for i in range(0,1000)]

    'omp parallel for schedule(dynamic,1) reduction(|:bit_or)'
    for i in range(0, 1000):
        bit_or = (bit_or | logics[i])

    if bit_or:
        print("E: reduction(|:bit_or)")
        result = False

    bit_or = 0;
    logics[1000//2]=1

    'omp parallel for schedule(dynamic,1) reduction(|:bit_or)'
    for i in range(0, 1000):
        bit_or = (bit_or | logics[i])

    if not bit_or:
        print("E: reduction(|:bit_or) with logics[1000/2]=1")
        result = False

    logics = [0 for i in range(0,1000)]

    'omp parallel for schedule(dynamic,1) reduction(^:exclusiv_bit_or)'
    for i in range(0, 1000):
        exclusiv_bit_or = (exclusiv_bit_or ^ logics[i])

    if exclusiv_bit_or:
        print("E: reduction(^:exclusiv_bit_or)")
        result = False

    exclusiv_bit_or = 0;
    logics[1000//2]=1

    'omp parallel for schedule(dynamic,1) reduction(^:exclusiv_bit_or)'
    for i in range(0, 1000):
        exclusiv_bit_or = (exclusiv_bit_or ^ logics[i])

    if not exclusiv_bit_or:
        print("E: reduction(^:exclusiv_bit_or) with logics[1000/2]=1")
        result = False

    max_ = 0;
    logics[1000//2]=1

    'omp parallel for schedule(dynamic,1) reduction(max:max_)'
    for i in range(0, 1000):
        max_ = max(max_, logics[i])

    if not max_:
        print("E: reduction(max:max_) with logics[1000/2]=1")
        result = False

    min_ = 1;
    logics = [1 for _ in range(1000)]
    logics[1000//2] = 0

    'omp parallel for schedule(dynamic,1) reduction(min:min_)'
    for i in range(0, 1000):
        min_ = min(min_, logics[i])

    if min_:
        print("E: reduction(min:min_) with logics[1000/2]=1")
        result = False
    return result



# unittest.skip Incorrect values computed on windows ...
# pythran export run(float, float, float, float, float, float, int, int, float[][])
# bench import numpy ; run(0,0,90,90, 1, 100, 80, 80, numpy.array([ [i/10., i/10., i/20.] for i in range(160)],dtype=numpy.double))
# runas import numpy ; run(0,0,90,90, 1, 100, 80, 80, numpy.array([ [i/10., i/10., i/20.] for i in range(80)],dtype=numpy.double))

import numpy as np


def run(xmin, ymin, xmax, ymax, step, range_, range_x, range_y, t):
    X, Y = t.shape
    pt = np.zeros((X, Y))
    "omp parallel for"
    for i in range(X):
        for j in range(Y):
            for k in t:
                tmp = 6368. * np.arccos(np.cos(xmin + step * i) * np.cos(k[0])
                                        * np.cos((ymin + step * j) - k[1])
                                        + np.sin(xmin + step * i)
                                        * np.sin(k[0]))
                if tmp < range_:
                    pt[i, j] += k[2] / (1+tmp)
    return pt



#from http://rosettacode.org/wiki/Read_a_file_line_by_line#Python
#pythran export readfile()
#runas readfile()

def readfile():
    return [line for line in open("pythran/tests/rosetta/read_conf.cfg")]



# from http://stackoverflow.com/questions/3939660/sieve-of-eratosthenes-finding-primes-python
# using a list instead of generator as return values

#pythran export primes_sieve(int)
#runas primes_sieve(100)
#bench primes_sieve(6000000)
def primes_sieve(limit):
    a = [True] * limit                          # Initialize the primality list
    a[0] = a[1] = False
    primes=list()

    for (i, isprime) in enumerate(a):
        if isprime:
            primes.append(i)
            for n in range(i*i, limit, i):     # Mark factors non-prime
                a[n] = False

    return primes



#runas solve(1000000)
# pythran export solve(int)
def solve(a):
    """
    The number, 197, is called a circular prime because all rotations of the
    digits: 197, 971, and 719, are themselves prime.

    There are thirteen such primes below 100: 2, 3, 5, 7, 11, 13, 17, 31, 37,
    71, 73, 79, and 97.

    How many circular primes are there below one million?
    """
    sieve = [True] * a
    sieve[0] = sieve[1] = False

    def mark(sieve, x):
        for i in range(x+x, len(sieve), x):
            sieve[i] = False

    for x in range(2, int(len(sieve) ** 0.5) + 1):
        mark(sieve, x)

    def circular(n):
        digits = []
        while n > 0:
            digits.insert(0, str(n % 10))
            n = n // 10
        for d in range(1, len(digits)):
            yield int(''.join(digits[d:] + digits[0:d]))

    count = 0
    for n, p in enumerate(sieve):
        if p:
            iscircularprime = 1
            for m in circular(n):
                if not sieve[m]:
                    iscircularprime = 0
                    break
            if iscircularprime:
                count = count + 1

    return count



# from http://stackoverflow.com/questions/19350395/python-jit-for-known-bottlenecks
# pythran export gf2mulmod(int, int, int)
# runas x, y, m = 2**10 , 2**6-1, 2**5-1; gf2mulmod(x, y, m)

def gf2mulmod(x,y,m):
    z = 0
    while x > 0:
        if (x & 1) != 0:
            z ^= y
        y <<= 1
        y2 = y ^ m
        if y2 < y:
            y = y2
        x >>= 1
    return z



import omp
from time import sleep


def omp_for_schedule_static_3():
    tmp_count = 1
    result = True
    chunk_size = 7
    tids = list(range(1001))
    notout = True
    maxiter = 0

    if 'omp parallel shared(tids)':
        if 'omp single':
            threads = omp.get_num_threads()

    if threads < 2:
        print("E: This test only works with at least two threads")
        return True

    tids[1000] = -1

    if 'omp parallel shared(tids)':
        tid = omp.get_thread_num()
        'omp for nowait schedule(static,chunk_size)'
        for j in range(1000):
            count = 0
            'omp flush(maxiter)'
            if j > maxiter:
                if 'omp critical':
                    maxiter = j

            while notout and count < 0.01 and maxiter == j:
                'omp flush(maxiter,notout)'
                sleep(0.0005)
                count += 0.0005

            tids[j] = tid

        notout = False
    lasttid = tids[0]
    tmp_count = 0
    for i in range(1001):
        if tids[i] == lasttid:
            tmp_count += 1
            continue

        if tids[i] == (lasttid + 1) % threads or tids[i] == -1:
            if tmp_count == chunk_size:
                tmp_count = 1
                lasttid = tids[i]
            else:
                if tids[i] == -1:
                    if i == 1000:
                        break;
                    else:
                        print("E: Last thread (thread with number -1) was " +
                              "found before the end.\n")
                        result = False
                else:
                    print("ERROR: chunk size was " + str(tmp_count) +
                    ". (assigned was " + str(chunk_size) + ")\n")
                    result = False
        else:
            print("ERROR: Found thread with number " + str(tids[i]) +
            " (should be inbetween 0 and " + str(threads - 1) + ").\n")
            result = False

    tids = list(range(1000))
    tids2 = list(range(1000))

    if 'omp parallel':
        'omp for schedule(static) nowait'
        for n in range(1000):
            if 1000 == n + 1:
                sleep(0.0005)
            tids[n] = omp.get_thread_num()
        'omp for schedule(static) nowait'
        for m in range(1, 1001):
            tids2[m-1] = omp.get_thread_num()

    for i in range(1000):
        if tids[i] != tids2[i]:
            print("E: Chunk no. " + str(i) + " was assigned once to thread " +
            str(tids[i]) + " and later to thread " + str(tids2[i]) + ".\n")
            result = False
    return result



#pythran export allpairs_distances(int)
#runas allpairs_distances(100)
#bench allpairs_distances(100)
import numpy as np

def dists(X,Y):
  return np.array([[np.sum( (x-y) ** 2) for x in X] for y in Y])

def allpairs_distances(d):
    #X = np.random.randn(1000,d)
    #Y = np.random.randn(200,d)
    X = np.arange(600*d).reshape((600,d))
    Y = np.arange(200*d).reshape((200,d))
    return dists(X,Y)



import omp
from time import sleep

def omp_for_schedule_static():
    NUMBER_OF_THREADS = 10
    CFSMAX_SIZE = 10000
    MAX_TIME = 0.01

    SLEEPTIME = 0.0005


    counter = 0
    tmp_count=1
    lastthreadsstarttid = -1
    result = 1
    chunk_size = 7

    tids = [0] * (CFSMAX_SIZE + 1)
    notout = 1
    maxiter = 0

    #omp parallel shared(tids,counter), num_threads(NUMBER_OF_THREADS)
    #omp single
    threads = omp.get_num_threads ()

    if threads < 2:
        print("This test only works with at least two threads");
        return 1
    tids[CFSMAX_SIZE] = -1
    if "omp parallel shared(tids) num_threads(NUMBER_OF_THREADS)":
        tid = omp.get_thread_num ();

        #omp for nowait schedule(static,chunk_size)
        for j in range(CFSMAX_SIZE):
            count = 0.
            #pragma omp flush(maxiter)
            if j > maxiter:
                #pragma omp critical
                maxiter = j
            while notout and (count < MAX_TIME) and (maxiter == j):
                #pragma omp flush(maxiter,notout)
                sleep (SLEEPTIME)
                count += SLEEPTIME
            tids[j] = tid

        notout = 0
        #omp flush(maxiter,notout)

    # analysing the data in array tids

    lasttid = tids[0]
    tmp_count = 0;

    print(lasttid)
    print(tids)
    for i in range(CFSMAX_SIZE):
        # If the work  was done by the same thread increase tmp_count by one.
        if tids[i] == lasttid:
            tmp_count+=1
            continue;

        # Check if the next thread had has the right thread number. When finding
        # threadnumber -1 the end should be reached.
        if (tids[i] == (lasttid + 1) % threads or tids[i] == -1):
            # checking for the right chunk size
            if (tmp_count == chunk_size):
                tmp_count = 1
                lasttid = tids[i]
            # If the chunk size was wrong, check if the end was reached 
            else:
                if (tids[i] == -1):
                    if (i == CFSMAX_SIZE):
                        print("Last thread had chunk size ", tmp_count)
                        break;
                    else:
                        print("ERROR: Last thread (thread with number -1) was found before the end.")
                        result = 0
                else:
                    print("ERROR: chunk size was . (assigned was )\n", tmp_count, chunk_size)
                    result = 0
        else:
            print("ERROR: Found thread with number %d (should be inbetween 0 and %d).", tids[i], threads - 1)
            result = 0;

    return result



def omp_for_schedule_auto():
    import omp
    sum = 0
    sum0 = 12345
    sum1 = 0
    if 'omp parallel private(sum1)':
        if 'omp single':
            threadsnum = omp.get_num_threads()
        'omp for firstprivate(sum0) schedule(auto)'
        for i in range(1, 1001):
            sum0 += i
            sum1 = sum0
        if 'omp critical':
            sum += sum1
    known_sum = 12345 * threadsnum + (1000 * (1000 + 1)) / 2
    return known_sum == sum



#unittest.skip threadprivate not supported
def omp_parallel_copyin():
    sum = 0
    sum1 = 7
    num_threads = 0

    if 'omp parallel copyin(sum1) private(i)':
        'omp for'
        for i in range(1, 1000):
            sum1 += i
        if 'omp critical':
            sum += sum1
            num_threads += 1

    known_sum = (999 * 1000) / 2 + 7 * num_threads
    return known_sum == sum



import unittest
from distutils.errors import CompileError
from pythran.tests import TestFromDir
import os
import pythran
from pythran.syntax import PythranSyntaxError
from pythran.spec import Spec

class TestOpenMP(TestFromDir):
    path = os.path.join(os.path.dirname(__file__), "openmp")

class TestOpenMP4(TestFromDir):
    path = os.path.join(os.path.dirname(__file__), "openmp.4")

    @staticmethod
    def interface(name, file=None):
        return Spec({name: []})

    @staticmethod
    def extract_runas(name, filepath):
        return ['#runas {}()'.format(name)]


class TestOpenMPLegacy(TestFromDir):
    '''
    Test old style OpenMP constructs, not using comments but strings
    and relying on function-scope locals
    '''
    path = os.path.join(os.path.dirname(__file__), "openmp.legacy")

    @staticmethod
    def interface(name, file=None):
        return Spec({name: []})

    @staticmethod
    def extract_runas(name, filepath):
        return ['#runas {}()'.format(name)]

# only activate OpenMP tests if the underlying compiler supports OpenMP
try:
    pythran.compile_cxxcode("omp", '#include <omp.h>',
                            extra_compile_args=['-fopenmp'],
                            extra_link_args=['-fopenmp'])
    import omp
    if '-fopenmp' in pythran.config.cfg.get('compiler', 'ldflags'):
        TestOpenMP4.populate(TestOpenMP4)
        TestOpenMP.populate(TestOpenMP)
        TestOpenMPLegacy.populate(TestOpenMPLegacy)
except PythranSyntaxError:
    raise
except (CompileError, ImportError):
    pass


if __name__ == '__main__':
    unittest.main()



#runas solve()
#pythran export solve()
def solve():
    '''
    The number 3797 has an interesting property. Being prime itself, it is possible to continuously remove digits from left to right, and remain prime at each stage: 3797, 797, 97, and 7. Similarly we can work from right to left: 3797, 379, 37, and 3.
    
    Find the sum of the only eleven primes that are both truncatable from left to right and right to left.
    
    NOTE: 2, 3, 5, and 7 are not considered to be truncatable primes.
    '''

    import math

    prime_list = [2, 3, 5, 7, 11, 13, 17, 19, 23]   # Ensure that this is initialised with at least 1 prime
    prime_dict = dict.fromkeys(prime_list, 1)

    def _isprime(n):
        ''' Raw check to see if n is prime. Assumes that prime_list is already populated '''
        isprime = n >= 2 and 1 or 0
        for prime in prime_list:                    # Check for factors with all primes
            if prime * prime > n: break             # ... up to sqrt(n)
            if not n % prime:
                isprime = 0
                break
        if isprime: prime_dict[n] = 1               # Maintain a dictionary for fast lookup
        return isprime

    def _refresh(x):
        ''' Refreshes primes upto x '''
        lastn = prime_list[-1]
        while lastn <= x:                           # Keep working until we've got up to x
            lastn = lastn + 1                       # Check the next number
            if _isprime(lastn):
                prime_list.append(lastn)            # Maintain a list for sequential access

    def prime(x):
        ''' Returns the xth prime '''

        lastn = prime_list[-1]
        while len(prime_list) <= x:                 # Keep working until we've got the xth prime
            lastn = lastn + 1                       # Check the next number
            if _isprime(lastn):
                prime_list.append(lastn)            # Maintain a list for sequential access
        return prime_list[x]

    digits = range(0, 10)
    prime_digits = (2, 3, 5, 7)


    def num(l):
        s = 0
        for n in l: s = s * 10 + n
        return s

    def is_left_truncatable(l):
        is_truncatable = 1
        for size in range(1, len(l)+1):
            n = num(l[:size])
            _refresh(int(math.sqrt(n)))
            if not _isprime(n):
                is_truncatable = 0
                break
        return is_truncatable

    def is_right_truncatable(l):
        is_truncatable = 1
        for size in range(0, len(l)):
            n = num(l[size:])
            _refresh(int(math.sqrt(n)))
            if not _isprime(n):
                is_truncatable = 0
                break
        return is_truncatable

    def gen(result, number):
        if len(number) > 6: return
        number = list(number)
        number.append(0)
        for digit in digits:
            number[-1] = digit
            if is_left_truncatable(number):
                if is_right_truncatable(number) and len(number) > 1:
                    result.append(num(number))
                gen(result, number)

    result = []
    gen(result, [])
    return sum(result)




#pythran export solve(int)
#runas solve(1000)
def solve(max):
    '''
    If we list all the natural numbers below 10 that are multiples of 3 or 5, we get 3, 5, 6 and 9. The sum of these multiples is 23.

    Find the sum of all the multiples of 3 or 5 below 1000.
    '''

    n = 0
    for i in range(1, max):
        if not i % 5 or not i % 3:
            n = n + i

    return n



#from http://rosettacode.org/wiki/Pangram_checker#Python
#pythran export ispangram(str)
#runas ispangram("The quick brown fox jumps over the lazy dog")
#runas ispangram("The brown fox jumps over the lazy dog")

import string

def ispangram(sentence, alphabet=string.ascii_lowercase):
    alphaset = set(alphabet)
    return alphaset <= set(sentence.lower())



#from http://rosettacode.org/wiki/Prime_decomposition#Python
#pythran export fac(int)
#runas fac(2**10 - 1)
#pythran export test_decompose(int)
#runas test_decompose(2**10 - 1)
import math

def decompose(n):
    primelist = [2, 3]
    for p in primes(primelist):
        if p*p > n:
            break
        while n % p == 0:
            yield p
            n //= p
    if n > 1:
        yield n

def test_decompose(to):
    return [i for i in decompose(to)]

def primes(primelist):
    for n in primelist: yield n

    n = primelist[-1]
    while True:
        n += 2
        for x in primelist:
            if not n % x: break
            if x * x > n:
                primelist.append(n)
                yield n
                break

def fac(n):
    step = lambda x: 1 + x * 4 - (x // 2) * 2
    maxq = math.floor(math.sqrt(n))
    d = 1
    q = n % 2 == 0 and 2 or 3
    while q <= maxq and n % q != 0:
        q = step(d)
        d += 1
    res = []
    if q <= maxq:
        res.extend(fac(n // q))
        res.extend(fac(q))
    else:
        res = [n]
    return res



#from http://rosettacode.org/wiki/Permutations_by_swapping#Python
#pythran export test()
#runas test()
#unittest.skip requires two-step typing

def s_permutations(seq):
    items = [[]]
    for j in seq:
        new_items = []
        for i, item in enumerate(items):
            if i % 2:
                # step up
                new_items += [item[:i] + [j] + item[i:]
                    for i in range(len(item) + 1)]
            else:
                # step down
                new_items += [item[:i] + [j] + item[i:]
                    for i in range(len(item), -1, -1)]
        items = new_items

    return [(tuple(item), -1 if i % 2 else 1)
        for i, item in enumerate(items)]

def test():
    for n in (3, 4):
        print('\nPermutations and sign of %i items' % n)
        for i in s_permutations(range(n)):
            print('Perm: ', i)



from pythran.tests import TestEnv
import unittest
import numpy as np
import pythran
from textwrap import dedent

from pythran.typing import List, Dict, NDArray, Tuple

class TestTyping(TestEnv):

    def test_index_dict_with_constant(self):
        code = 'def index_dict_with_constant(d): return d[0]'
        return self.run_test(code, {0:2}, index_dict_with_constant=[Dict[int,int]])

    def test_module_bad_attribute(self):
        code = 'def module_bad_attribute(): import random as m; return m.real'

        with self.assertRaises(pythran.syntax.PythranSyntaxError):
            pythran.compile_pythrancode("dumbo", code, pyonly=True)

    def test_module_bad_call(self):
        code = 'def module_bad_call(): import numpy.int32 as i32; return i32()'

        with self.assertRaises(pythran.syntax.PythranSyntaxError):
            pythran.compile_pythrancode("dumbo", code, pyonly=True)

    def test_module_invalid_sequence_mult(self):
        code = 'def foo(x): return [x] * 3.'

        with self.assertRaises(pythran.syntax.PythranSyntaxError):
            pythran.compile_pythrancode("dumbo", code, pyonly=True)

    def test_immutable_default0(self):
        code = 'def immutable_default0(x=[1]): pass'

        with self.assertRaises(pythran.syntax.PythranSyntaxError):
            pythran.compile_pythrancode("dumbo", code, pyonly=True)

    def test_undefinied_variable_in_test(self):
        code = 'def undefinied_variable_in_test(x):\n if x: print(A)'

        with self.assertRaises(pythran.syntax.PythranSyntaxError):
            pythran.compile_pythrancode("dumbo", code, pyonly=True)

    def test_immutable_default1(self):
        code = 'def immutable_default1(x={1}): pass'

        with self.assertRaises(pythran.syntax.PythranSyntaxError):
            pythran.compile_pythrancode("dumbo", code, pyonly=True)

    def test_immutable_default2(self):
        code = 'def immutable_default2(x=1): pass'
        pythran.compile_pythrancode("dumbo", code, pyonly=True)

    def test_immutable_default3(self):
        code = 'def immutable_default3(x=(1, 2)): pass'
        pythran.compile_pythrancode("dumbo", code, pyonly=True)

    def test_immutable_default4(self):
        code = 'def immutable_default4(x=None): pass'
        pythran.compile_pythrancode("dumbo", code, pyonly=True)

    def test_immutable_default5(self):
        code = 'def immutable_default5(x=True): pass'
        pythran.compile_pythrancode("dumbo", code, pyonly=True)

    def test_immutable_default6(self):
        code = 'g=1\ndef immutable_default6(x=g): pass'
        pythran.compile_pythrancode("dumbo", code, pyonly=True)

    def test_immutable_default7(self):
        code = 'def g(): pass\ndef immutable_default7(x=g): pass'
        pythran.compile_pythrancode("dumbo", code, pyonly=True)

    def test_immutable_default8(self):
        code = 'def g(): pass\ndef immutable_default8(x=int): pass'
        pythran.compile_pythrancode("dumbo", code, pyonly=True)

    def test_list_of_set(self):
        code = '''
def list_of_set():
    l=[set()]
    l[0].add("12")
    return l'''
        self.run_test(code, list_of_set=[])

    def test_dict_of_set(self):
        code = '''
def dict_of_set():
    l={0:set()}
    l[0].add("12")
    return l'''
        self.run_test(code, dict_of_set=[])

    def test_typing_aliasing_and_indices(self):
        self.run_test('def typing_aliasing_and_indices(i): d={};e={}; f = e or d; f[1]=i; return d,e,f', 118, typing_aliasing_and_indices=[int])

    def test_typing_aliasing_and_combiner(self):
        self.run_test('def typing_aliasing_and_combiner(i): d=set();e=set(); f = e or d; f.add(i); return d,e,f', 117, typing_aliasing_and_combiner=[int])

    def test_typing_aliasing_and_combiner_back(self):
        self.run_test('def typing_aliasing_and_combiner_back(i): d=set();e=set(); f = e or d; e.add(i); return d,e,f', 116, typing_aliasing_and_combiner_back=[int])

    def test_typing_aliasing_and_fwd(self):
        self.run_test('def typing_aliasing_and_fwd(i): fwd = lambda x:x; l = []; fwd(l).append(i); return l', 115, typing_aliasing_and_fwd=[int])

    def test_typing_aliasing_and_constant_subscript(self):
        code = '''
        def typing_aliasing_and_constant_subscript(i):
            a=[]
            b=(a,)
            b[0].append(i)
            return a, b
        '''
        self.run_test(code, 118, typing_aliasing_and_constant_subscript=[int])

    def test_typing_aliasing_and_variable_subscript(self):
        code = '''
        def typing_aliasing_and_variable_subscript(i):
            a=[]
            b=[a]
            b[i].append(i)
            return a, b
        '''
        self.run_test(code, 0, typing_aliasing_and_variable_subscript=[int])

    def test_typing_aliasing_and_variable_subscript_combiner(self):
        code = '''
        def typing_aliasing_and_variable_subscript_combiner(i):
            a=[list.append, lambda x,y: x.extend([y])]
            b = []
            a[i](b, i)
            return b
        '''
        self.run_test(code, 1, typing_aliasing_and_variable_subscript_combiner=[int])

    def test_typing_and_function_dict(self):
        code = '''
            def typing_and_function_dict(a):
                funcs = {
                    'zero' : lambda x: x.add(0),
                    'one' : lambda x: x.add(1),
                }
                s = set()
                funcs[a](s)
                return s
        '''
        self.run_test(code, 'one', typing_and_function_dict=[str])

    def test_typing_and_iterate_over_function_list(self):
        code = '''
            def typing_and_iterate_over_function_list():
                funcs = [
                    lambda x: x.add(0),
                    lambda x: x.add(2),
                ]
                s = set()
                for f in funcs:
                    f(s)
                return s
        '''
        self.run_test(code, typing_and_iterate_over_function_list=[])


    def test_typing_aliasing_and_update(self):
        code = '''
def foo(d):
    f=d
    f+=[1]
def typing_aliasing_and_update():
    a= []
    foo(a)
    return a'''
        self.run_test(code, typing_aliasing_and_update=[])

    def test_typing_aliasing_and_update_and_globals(self):
        code = '''
def f(l): return len(l)
def g(l): l.append(1) ; return 0
foo=[f,g]
def typing_aliasing_and_update_and_globals(i):
    h = []
    return foo[i](h), h'''
        self.run_test(code, 1, typing_aliasing_and_update_and_globals=[int])

    def test_typing_aliasing_and_update_and_multiple_aliasing0(self):
        code = '''
def f(l): return len(l)
def g(l): l.append(1) ; return 0
def foo(i):
    if i < 1:
        return f
    else:
        return g
def typing_aliasing_and_update_and_multiple_aliasing0(i):
    h = []
    return foo(i)(h), h'''
        self.run_test(code, 1, typing_aliasing_and_update_and_multiple_aliasing0=[int])

    def test_typing_aliasing_and_update_and_multiple_aliasing1(self):
        code = '''
def foo(i):
    if i < 1:
        return list.remove
    else:
        return list.append
def typing_aliasing_and_update_and_multiple_aliasing1(i):
    h = []
    foo(i)(h, 1)
    return h'''
        self.run_test(code, 1, typing_aliasing_and_update_and_multiple_aliasing1=[int])


    def test_functional_variant_assign0(self):
        code='''
def functional_variant_assign0(n):
    if n > 3:
        x = "e" * n
        f = lambda y: x + y
    else:
        x = "g" * n
        f = lambda y: x + "er"
    return f("re")'''
        self.run_test(code, 12, functional_variant_assign0=[int])

    def test_functional_variant_assign1(self):
        code='''
def functional_variant_assign1(n):
    if n > 3:
        x = "e" * n
        f = lambda y: x + y
    else:
        x = "g" * n
        f = lambda y: x + "er"
    return f("re")'''
        self.run_test(code, 1, functional_variant_assign1=[int])

    def test_functional_variant_assign2(self):
        code='''
def functional_variant_assign2(n):
    if n > 3:
        f = lambda x: (1, x * 1.j)
    else:
        f = lambda x: (x * 1.j, 1)
    return sum(f(3))'''
        self.run_test(code, 1, functional_variant_assign2=[int])

    def test_functional_variant_container0(self):
        code='''
import math
def functional_variant_container0(i):
    l=[]
    l.append(math.cos)
    l.append(math.sin)
    return l[i](12)'''
        self.run_test(code, 0, functional_variant_container0=[int])

    def test_functional_variant_container1(self):
        code='''
import math
def functional_variant_container1(i):
    l=[math.cos, math.sin]
    return l[i](12)'''
        self.run_test(code, 1, functional_variant_container1=[int])

    def test_functional_variant_container2(self):
        code='''
import math
l = [math.cos, math.sin, math.asin, math.acos, math.sqrt]
def functional_variant_container2(i):
    return l[i](1.)'''
        self.run_test(code, 4, functional_variant_container2=[int])

    @unittest.skip("bad typing: need backward propagation")
    def test_type_set_in_loop(self):
        code = '''
def type_set_in_loop():
    a = [[]]
    for i in range(2):
        b = []
        for j in a:
            b += [j] + [[1]]
        a = b
    return a,b'''
        self.run_test(code, type_set_in_loop=[])

    @unittest.skip("bad typing: need backward propagation")
    def test_type_set_in_while(self):
        code = '''
def type_set_in_while():
    a = [[]]
    n = 3
    while n:
        b = []
        for j in a:
            b += [j] + [[1]]
        a = b
        n -= 1
    return a,b'''
        self.run_test(code, type_set_in_while=[])

    def test_recursive_interprocedural_typing0(self):
        code = '''
from cmath import exp, pi

def fft(x):
    N = len(x)
    if N <= 1: return x
    even = fft(x[0::2])
    odd =  fft(x[1::2])
    return [even[k] + exp(-2j*pi*k/N)*odd[k] for k in range(N//2)] + \
           [even[k] - exp(-2j*pi*k/N)*odd[k] for k in range(N//2)]

def recursive_interprocedural_typing0(c):
   import numpy as np
   l = [1.0, 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, c]
   z = np.abs(fft(l))
   return z'''
        self.run_test(code, 0.0, recursive_interprocedural_typing0=[float])

    def test_recursive_interprocedural_typing1(self):
        code = '''
def s_perm(seq):
    if not seq:
        return [[]]
    else:
        new_items = []
        for item in s_perm(seq[:-1]):
            new_items += [item + seq for i in range(1)]
        return new_items
def recursive_interprocedural_typing1(c):
    l = [1,2] * c
    return s_perm(l)'''
        self.run_test(code, 3, recursive_interprocedural_typing1=[int])

    @unittest.skip("bad typing: recursion and specialized list type")
    def test_recursive_interprocedural_typing2(self):
        code = '''
def s_perm(seq):
    if not seq:
        return [[]]
    else:
        new_items = []
        for item in s_perm(seq[:-1]):
            new_items += [item + seq for i in range(1)]
        return new_items
def recursive_interprocedural_typing2(c):
    l = [1,2,c]
    return s_perm(l)'''
        self.run_test(code, 3, recursive_interprocedural_typing2=[int])

    def test_print_numpy_types(self):
        self.run_test('''
            import numpy
            def print_type(t): print(t)
            def print_numpy_types(n):
                print_type(numpy.ones(n, dtype=bool).dtype)
                print_type(numpy.ones(n, dtype=int).dtype)
                print_type(numpy.ones(n, dtype=complex).dtype)
                print_type(numpy.ones(n, dtype=float).dtype)
                print_type(numpy.ones(n, dtype=numpy.uint8).dtype)
                print_type(numpy.ones(n, dtype=numpy.uint16).dtype)
                print_type(numpy.ones(n, dtype=numpy.uint64).dtype)
                print_type(numpy.ones(n, dtype=numpy.double).dtype)
                print_type(numpy.ones(n, dtype=complex).dtype)
            ''',
            3,
            print_numpy_types=[int])

    def test_constant_argument_variant_functor0(self):
        self.run_test('''
            def foo(x): x[0] = 0
            def bar(x): x[1] = 1
            l = [foo, bar]
            def constant_argument_variant_functor0(i):
                x = [-1, -1]
                l[i](x)
                return x''',
            0,
            constant_argument_variant_functor0=[int])

    def test_constant_argument_variant_functor1(self):
        self.run_test('''
            def foo(x): x[0] = 0
            def bar(x): x[1] = 1
            l = [foo, bar]
            def constant_argument_variant_functor1(i):
                x = [i, i]
                [f(x) for f in l]
                return x''',
            -1,
            constant_argument_variant_functor1=[int])

    def test_slice_assign(self):
        self.run_test('''
            import numpy as np
            def slice_assign (M):
                errs = np.zeros (M)
                errs[0] = 1
                errs[:] = 0
                return errs''',
            4,
            slice_assign=[int])

    def verify_type_error(self, code):
        with self.assertRaises(pythran.types.tog.PythranTypeError):
            _, eh = pythran.generate_cxx("dumbo", dedent(code))
            eh()

    def test_type_inference0(self):
        code = '''
        def wc(content):
            d = {}

            for word in content.split():
                d[word] = d.get(word, 0) + 1

            # Use list comprehension
            l = [(freq, word) for word, freq in d.items()]

            return sorted(l)
        '''
        self.run_test(code, "cha-la head cha-la", wc=[str])

        code_bis = code.replace("1", "'1'")
        self.verify_type_error(code_bis)

        code_ter = code.replace("0", "None")
        self.verify_type_error(code_ter)

    def test_type_inference1(self):
        code = '''
            def invalid_augassign(n):
                s = n + "1"
                s += 2
                return s'''
        self.verify_type_error(code)


    def test_type_inference2(self):
        code = '''
            def invalid_ifexp(n):
                return 1 if n else "1"'''
        self.verify_type_error(code)


    def test_type_inference3(self):
        code = '''
            def invalid_unary_op(n):
                return -(n + 'n')'''
        self.verify_type_error(code)


    def test_type_inference4(self):
        code = '''
            def invalid_list(n):
                return [n, len(n)]'''
        self.verify_type_error(code)


    def test_type_inference5(self):
        code = '''
            def invalid_set(n):
                return {n, len(n)}'''
        self.verify_type_error(code)

    def test_type_inference6(self):
        code = '''
            def invalid_dict_key(n):
                return {n:1, len(n):2}'''
        self.verify_type_error(code)


    def test_type_inference7(self):
        code = '''
            def invalid_dict_value(n):
                return {1:n, 2:len(n)}'''
        self.verify_type_error(code)

    def test_type_inference8(self):
        code = '''
            def invalid_multi_return(n):
                for i in n:
                    return [n]
                return {n}'''
        self.verify_type_error(code)

    def test_type_inference9(self):
        code = '''
            def invalid_multi_yield(n):
                for i in n:
                    yield [n]
                yield n'''
        self.verify_type_error(code)


    def test_type_inference10(self):
        code = '''
            def valid_augassign(l):
                l *= 0
                return l[1:2]'''
        return self.run_test(code, np.array([0,1,2,3,4]), valid_augassign=[NDArray[int, :]])

    def test_type_inference11(self):
        code = '''
            def valid_tuple_index(l):
                return (1, 2, 3, 4)[l]'''
        return self.run_test(code, 0, valid_tuple_index=[int])

    def test_exact_float_check(self):
        code = 'def exact_float_check(i): return i'
        return self.run_test(code, np.float64(1.1), exact_float_check=[float])

    def test_exact_complex_check(self):
        code = 'def exact_complex_check(i): return i'
        return self.run_test(code, np.complex128(2), exact_complex_check=[complex])

    def test_alias_update_in_loop_and_test(self):
        code = '''
            def alias_update_in_loop_and_test(X,f):
                for i in range(2):
                    if i==0: A = f*X[:,i]
                    else: A+=f*X[:,i]
                return A'''
        return self.run_test(code, np.arange(9).reshape(3,3), 3, alias_update_in_loop_and_test=[NDArray[int,:,:], int])

    def test_alias_update_in_multiple_opaque_tests(self):
        code = '''
            def alias_update_in_multiple_opaque_tests(X,f):
                A = X[:, 0]
                if f > 3:
                    A = (f * X[:, 0])
                else:
                    A += (f * X[:, 0])
                pass
                if f <= 3:
                    A = (f * X[:, 1])
                else:
                    A += (f * X[:, 1])
                return A'''

        return self.run_test(code, np.arange(9).reshape(3,3), 9, alias_update_in_multiple_opaque_tests=[NDArray[int,:,:], int])

    def test_alias_update_in_multiple_different_opaque_tests(self):
        code = '''
            def alias_update_in_multiple_different_opaque_tests(f):
                if f > 3:
                    A = 1
                else:
                    A = 2

                if f <= 3:
                    A = "e" * A
                else:
                    A = "f" * A
                return A'''

        return self.run_test(code, 9, alias_update_in_multiple_different_opaque_tests=[int])

    def test_lambda_partial_merge(self):
        code = '''
            def lambda_partial_merge(a, c, s):
              if s == "a":
                x = lambda y:y 
              else:
                x = lambda y:y+c
              return x(a)'''
        return self.run_test(code, 1, 2, "A", lambda_partial_merge=[int, int, str])

    def test_unpacking_aliasing(self):
        code = '''
            def effect(c, v):
              a, _ = c
              a[0] += v
            def unpacking_aliasing(v, c):
              effect(c, v)
              a, _ = c
              return a'''
        return self.run_test(code, 2, ([10, 20], [30]), unpacking_aliasing=[int, Tuple[List[int], List[int]]])

    @unittest.skip("bad typing: need higher order function handling")
    def test_higher_order0(self):
        code = '''
            def b(x, y):
                x(y, 1)

            def higher_order0(n):
                t = []
                foo = list.append
                b(foo, t)
                return t'''
        return self.run_test(code, 3, higher_order0=[int])

    @unittest.skip("bad typing: need higher order function handling")
    def test_higher_order1(self):
        code = '''
            def b(x, y):
                x(y, 1)

            def higher_order0(n):
                t = []
                def foo(g):
                    t.append(g)
                b(foo, t)
                return t'''
        return self.run_test(code, 3, higher_order0=[int])

    def test_rvalue_type_update_list(self):
        code = '''
          def rvalue_type_update_list(x):
              def foo(x): x.append(1.5); return x
              return foo([x])'''
        return self.run_test(code, 3, rvalue_type_update_list=[int])

    def test_rvalue_type_update_set(self):
        code = '''
          def rvalue_type_update_set(x):
              def foo(x): x.add(1.5); return x
              return foo({x})'''
        return self.run_test(code, 3, rvalue_type_update_set=[int])

    def test_rvalue_type_update_dict(self):
        code = '''
          def rvalue_type_update_dict(x):
              def foo(x): x[1.5] = 1.5; return x
              return foo({x:x})'''
        return self.run_test(code, 3, rvalue_type_update_dict=[int])

    def test_numpy_array_combiner0(self):
        code = '''
            import numpy as np
            def numpy_array_combiner0(n):
                if n > 10:
                    return np.ones(10)
                else:
                    return np.ones(n)'''
        return self.run_test(code, 3, numpy_array_combiner0=[int])

    def test_numpy_array_combiner1(self):
        code = '''
            import numpy as np
            def numpy_array_combiner1(n):
                if n <= 10:
                    return np.ones(10)
                else:
                    return np.ones(n)'''
        return self.run_test(code, 3, numpy_array_combiner1=[int])

    def test_numpy_array_combiner2(self):
        code = '''
            import numpy as np
            def numpy_array_combiner2(n):
                if n > 10:
                    return np.ones(10)
                else:
                    return np.arange(n)'''
        return self.run_test(code, 3, numpy_array_combiner2=[int])

    def test_numpy_array_combiner3(self):
        code = '''
            import numpy as np
            def numpy_array_combiner3(n):
                if n <= 10:
                    return np.ones(10)
                else:
                    return np.arange(n)'''
        return self.run_test(code, 3, numpy_array_combiner3=[int])

    def test_numpy_array_combiner4(self):
        code = '''
            import numpy as np
            def numpy_array_combiner4(n):
                if n > 10:
                    return np.ones(10)
                else:
                    return np.arange(n)[1:n//2]'''
        return self.run_test(code, 6, numpy_array_combiner4=[int])

    def test_numpy_array_combiner5(self):
        code = '''
            import numpy as np
            def numpy_array_combiner5(n):
                if n <= 10:
                    return np.ones(10)
                else:
                    return np.arange(n)[1:-1]'''
        return self.run_test(code, 6, numpy_array_combiner5=[int])

    def test_numpy_array_in_tuple0(self):
        code = '''
import numpy as np


def Test_reset(self, P, Q):
    print('In SHAPE', P, Q)
    self[0][0] = np.zeros((P, Q))
    print('Out SHAPE', self[0][0].shape)
    return self

def numpy_array_in_tuple0(n):
    return Test_reset(([np.zeros((1, 1))],), 64, n)'''
        return self.run_test(code, 6, numpy_array_in_tuple0=[int])

    def test_numpy_array_in_tuple1(self):
        code = '''
import numpy as np

def CL___init__(N):
    self_X = np.zeros((0, 0))
    return [self_X]

def CL_test(self):
    self[0] = np.zeros((3, 3))
    Y = []
    X = self[0]
    for ii in range(2):
        Y.append(X[:, ii:ii+1])
    Y = np.stack(Y, axis=0)
    return(Y)

def numpy_array_in_tuple1(N = 10):
    CL = CL___init__(N)
    return CL_test(CL)'''
        return self.run_test(code, 6, numpy_array_in_tuple1=[int])



#runas solve(1000)
#pythran export solve(int)
'''
Euler published the remarkable quadratic formula:

n^2 + n + 41

It turns out that the formula will produce 40 primes for the consecutive values n = 0 to 39. However, when n = 40, 402 + 40 + 41 = 40(40 + 1) + 41 is divisible by 41, and certainly when n = 41, 41^2 + 41 + 41 is clearly divisible by 41.

Using computers, the incredible formula  n^2 - 79n + 1601 was discovered, which produces 80 primes for the consecutive values n = 0 to 79. The product of the coefficients, -79 and 1601, is -126479.

Considering quadratics of the form:

n^2 + an + b, where |a| <= 1000 and |b| <= 1000

where |n| is the modulus/absolute value of n
e.g. |11| = 11 and |4| = 4
Find the product of the coefficients, a and b, for the quadratic expression that produces the maximum number of primes for consecutive values of n, starting with n = 0.

'''

def solve(edge):
 prime_list = [2, 3, 5, 7, 11, 13, 17, 19, 23]   # Ensure that this is initialised with at least 1 prime
 prime_dict = dict.fromkeys(prime_list, 1)

 def _isprime(n):
     ''' Raw check to see if n is prime. Assumes that prime_list is already populated '''
     isprime = n >= 2 and 1 or 0
     for prime in prime_list:                    # Check for factors with all primes
         if prime * prime > n: break             # ... up to sqrt(n)
         if not n % prime:
             isprime = 0
             break
     if isprime: prime_dict[n] = 1               # Maintain a dictionary for fast lookup
     return isprime

 def _refresh(x):
     ''' Refreshes primes upto x '''
     lastn = prime_list[-1]
     while lastn <= x:                           # Keep working until we've got up to x
         lastn = lastn + 1                       # Check the next number
         if _isprime(lastn):
             prime_list.append(lastn)            # Maintain a list for sequential access

 def prime(x):
     ''' Returns the xth prime '''

     lastn = prime_list[-1]
     while len(prime_list) <= x:                 # Keep working until we've got the xth prime
         lastn = lastn + 1                       # Check the next number
         if _isprime(lastn):
             prime_list.append(lastn)            # Maintain a list for sequential access
     return prime_list[x]

 max_pair = (0,0,0)
 for a in range(-1 * edge + 1, edge):
     for b in range(max(2, 1-a), edge): # b >= 2, a + b + 1 >= 2
         n, count = 0, 0
         while True:
             v = n*n + a*n + b
             _refresh(v)
             if _isprime(v): count = count + 1
             else: break
             n = n + 1
         if count > max_pair[2]:
             max_pair = (a,b,count)

 return max_pair[0] * max_pair[1]



""" Immediates gathers immediates. For now, only integers within shape are
considered as immediates """

import gast as ast
from pythran.tables import MODULES
from pythran.analyses import Aliases
from pythran.passmanager import NodeAnalysis
from pythran.utils import pythran_builtin, isnum

_make_shape = pythran_builtin('make_shape')


class Immediates(NodeAnalysis):
    def __init__(self):
        self.result = set()
        super(Immediates, self).__init__(Aliases)

    def visit_Call(self, node):
        func_aliases = self.aliases[node.func]
        for alias in func_aliases:
            if getattr(alias, "immediate_arguments", []):
                for i, arg in enumerate(node.args):
                    if i in alias.immediate_arguments:
                        if isinstance(arg, ast.Constant):
                            self.result.add(arg)

        if len(func_aliases) == 1 and next(iter(func_aliases)) is _make_shape:
            self.result.update(a for a in node.args
                               if isnum(a)
                               and isinstance(a.value, int)
                               and a.value >= 0)
            return

        return self.generic_visit(node)



#pythran export histogram(float list, int)
#runas histogram([ (i*1.1+j*2.3)%10 for i in range(100) for j in range(100) ],10)
#bench histogram([ (i*1.1+j*2.3)%10 for i in range(1000) for j in range(2000) ],10)
def histogram(data, bin_width):
    lower_bound, upper_bound = min(data), max(data)
    out_data=[0]*(1+bin_width)
    for i in data:
        out_data[ int(bin_width * (i - lower_bound) / ( upper_bound - lower_bound)) ]+=1
    out_data[-2]+=out_data[-1]
    out_data.pop()
    return out_data



from pythran.tests import TestEnv
from unittest import skip
from pythran.typing import List, Dict, NDArray
import pythran
import numpy as np

class TestNone(TestEnv):

    def test_returned_none(self):
        code = '''
def dummy(l):
    if l: return None
    else: return l
def returned_none(a):
    return dummy(a)'''
        self.run_test(code, [1, 2], returned_none=[List[int]])

    def test_eq_neq_scalar(self):
        code = 'def eq_neq_scalar(x): return x == 1, x!= 1'
        self.run_test(code, 1, eq_neq_scalar=[int])

    def test_eq_neq_nonscalar(self):
        code = 'def eq_neq_nonscalar(x): return x == [1], x!= [1]'
        self.run_test(code, [1], eq_neq_nonscalar=[List[int]])

    def test_returned_none_member(self):
        code = '''
def dummy(l):
    if not l: return None
    else: return l
def returned_none_member(a):
    t = dummy(a)
    if t is None:
        return None
    else:
        return t.count(1)'''
        self.run_test(code, [1, 2], returned_none_member=[List[int]])

    def test_use_none(self):
        code = """
            def none_input(a):
                l = list(range(10))
                if a is None:
                    return l[a:8]
                else:
                    return l[a:3]
            """
        self.run_test(code, None, none_input=[type(None)])

    def test_is_none_default(self):
        code = '''
        import numpy as np
        def is_none_default(sr,N):
            C = test2(sr, N)
            return C

        def test2(sr, N,fmax=None):
            if fmax is None:
                fmax = float(sr) / 2
            A = np.arange(1.+N/2)
            B = np.arange(1.+fmax/2)
            return A[0] - B[0]'''
        self.run_test(code, 1, 3, is_none_default=[int, int])

    def test_is_none_return_None(self):
        code = '''
import numpy as np
def is_none_return_None(signal=None):
    if signal is None:
        return
    signal[0,] += 1.2'''
        self.run_test(code, np.array([1.]), is_none_return_None=[NDArray[float, :]])

    def test_is_none_return_None_bis(self):
        code = '''
import numpy as np
def is_none_return_None_bis(signal=None):
    if signal is None:
        return
    signal[0,] += 1.2'''
        self.run_test(code, is_none_return_None_bis=[])

    def test_return_in_true(self):
        code = '''
        def return_in_true(x):
            return helper(x, x), helper(x, None)

        def helper(x, y):
            if y == None:
                return x
            return x + 1'''
        self.run_test(code, 1, return_in_true=[int])

    def test_return_in_false(self):
        code = '''
        def return_in_false(x):
            return helper(x, x), helper(x, None)

        def helper(x, y):
            if y != None:
                x += y
            else:
                return x
            return x + 1'''
        self.run_test(code, 1, return_in_false=[int])

    def test_potential_return_in_true(self):
        code = '''
        def potential_return_in_true(x):
            return helper(0, x), helper(x, None), helper(18, x)

        def helper(x, y):
            if None != y:
                if y > 1:
                    return 5
                else:
                    x += y
            else:
                return x
            return x + 1'''
        self.run_test(code, 1, potential_return_in_true=[int])

    def test_potential_return_in_false(self):
        code = '''
        def potential_return_in_false(x):
            return helper(0, x), helper(x, None), helper(18, x)

        def helper(x, y):
            if None is not y:
                x += y
            else:
                if x > 1:
                    return x
                else:
                    return 7
            return x + 1'''
        self.run_test(code, 1, potential_return_in_false=[int])

    def test_and_is_none(self):
        code = '''
        def and_is_none(x):
            return helper(0, x), helper(x, None), helper(18, x)

        def helper(x, y):
            if None is not y and x is not None:
                x += y
            else:
                return 7
            return x + 1'''
        self.run_test(code, 1, and_is_none=[int])

    def test_multiple_is_none(self):
        code = '''
        def multiple_is_none(x):
            return helper(0, x), helper(x, None), helper(18, x)

        def helper(x, y):
            if not (x is None is y):
                x += y
            else:
                return 7
            return x + 1'''
        with self.assertRaises(pythran.syntax.PythranSyntaxError):
            self.run_test(code, 1, multiple_is_none=[int])

    def test_different_return(self):
        code = '''
        def different_return(x):
            return helper(0, x), helper(x, None), helper(18, x)

        def helper(x, y):
            if not (x is None is y):
                return "7"
            else:
                return 7
            return x + 1'''
        with self.assertRaises(pythran.syntax.PythranSyntaxError):
            self.run_test(code, 1, different_return=[int])

    def test_is_none_in_expr(self):
        code = '''
        def is_none_in_expr(x):
            return (1 if None == x else 2), (x if None is x else 1), (1 if None != x else 2), (x if None is not x else 1)
        '''
        self.run_test(code, 1, is_none_in_expr=[int])

    def test_none_bool_tuple(self):
        code = '''
        def none_bool_tuple(x):
            if x > 2:
                y = (1, '2')
            else:
                y = None
            if x > 2:
                z = ()
            else:
                z = None
            return bool(y), bool(z)'''
        self.run_test(code, 1, none_bool_tuple=[int])

    def test_none_large_cond(self):
        code = '''
        def none_large_cond(x):
            return helper(x, None, None), helper(None, x, None), helper(None, None, x)

        def helper(x, y, z):
            if x is None or y is None and z is None:
                return 5
            else:
                return 6'''
        self.run_test(code, 1, none_large_cond=[int])

    def test_none_mixed_test0(self):
        code = '''
        def none_mixed_test0(x):
            return helper(x, 1), helper(x, 3)

        def helper(x, y):
            if x is None or x > 2:
                return 5
            if y > 0 or x is not None:
                return 6'''
        self.run_test(code, 1, none_mixed_test0=[int])

    def test_none_mixed_test1(self):
        code = '''
        def none_mixed_test1(x):
            return helper(x, 1), helper(x, 3)

        def helper(x, y):
            if x is not None and x > 2 and x < 8:
                return 5
            if y > 0 and x is not None:
                return 6'''
        self.run_test(code, 1, none_mixed_test1=[int])

    def test_none_mixed_test2(self):
        code = '''
        def none_mixed_test2(x):
            return helper(x, 1), helper(x, 3)

        def helper(x, y):
            if x is not None and x > 2 or x < 8:
                return 5
            if y > 0 or x is None:
                return 6'''
        self.run_test(code, 1, none_mixed_test2=[int])

    def test_none_mixed_test3(self):
        code = '''
        def none_mixed_test3(x):
            return helper(x, 1), helper(x, 3)

        def helper(x, y):
            return 5 if x is None or x > 2 or x < 8 else 6'''
        self.run_test(code, 1, none_mixed_test3=[int])

    def test_none_mixed_test4(self):
        code = '''
        def none_mixed_test4(x):
            return helper(x, 4), helper(x, 3)

        def helper(x, y):
            return 5 if x is not None and x > 2 and x < 8 else 6'''
        self.run_test(code, 4, none_mixed_test4=[int])

    def test_none_mixed_test5(self):
        code = '''
        def none_mixed_test5(x):
            return helper(x, 1), helper(x, 3)

        def helper(x, y):
            return 5 if x is not None and x > 5 or x < 8 else 6'''
        self.run_test(code, 1, none_mixed_test5=[int])

    def test_987(self):
        code = '''
        def test_987(x):
            a = None
            if x == 5:
                a = 3
            if a is not None:
                return 1
            return 0'''
        self.run_test(code, 1, test_987=[int])
    def test_987_0(self):
        code = '''
        def test_987_0(x):
            a = None
            if x == 5:
                a = 3
            if a is not None:
                return 1
            return 0'''
        self.run_test(code, 5, test_987_0=[int])
    def test_987_1(self):
        code = '''
        def test_987_1(x):
            a = None
            b = 0
            if x != 12:
                a = x
            if a is not None and a < 5:
                b = 1
            return (b, a)'''
        self.run_test(code, 12, test_987_1=[int])
    def test_987_2(self):
        code = '''
        def test_987_2(x):
            a = None
            b = 0
            if x != 12:
                a = x
            if a is not None and a < 5:
                b = 1
            return (b, a)'''
        self.run_test(code, 13, test_987_2=[int])
    def test_987_3(self):
        code = '''
        def test_987_3(x):
            a = None
            b = 0
            if x != 12:
                a = x
            if a is not None and a < 5:
                b = 1
            return (b, a)'''
        self.run_test(code, 3, test_987_3=[int])

    def test_none_to_str(self):
        code = """
        def dump_as_str(x):
            y = x
            return str(y) + str(y) * 2
        def none_to_str(x):
            if x > 1:
                y = None
            else:
                y = 2
            return dump_as_str(None), dump_as_str(y)"""
        self.run_test(code, 0, none_to_str=[int])

    def test_is_none_attribute(self):
        code = '''
            def is_none_attribute(x):
                a = [x, x]
                if x is not None:
                    a.pop()
                return a'''
        self.run_test(code, 0, is_none_attribute=[int])

    def test_none_combine_tuple(self):
        code = '''
            def none_combine_tuple(n):
                def get_header(name):
                    if name:
                        return 1
                    else:
                        return None

                a = None

                lpix = get_header(n)

                # Don't work
                if lpix is None:
                    a = 3

                return a'''
        self.run_test(code, 'hello', none_combine_tuple=[str])

    def test_none_tuple(self):
        code = '''
            def test_none_tuple(i):
                a = 0
                b = 0
                c = 0
                h = None
                if i % 2 == 0:
                    h = (1, 2, 3)
                if h is not None:
                    a, b, c = h
                return a'''
        self.run_test(code, 10, test_none_tuple=[int])

    def test_none_loop_break(self):
        code = '''
            def none_loop_break(n):
                x = [None if i%2 else 1 for i in range(n)]
                s = 3
                for j in x:
                    if j is None:
                        break
                    s += 1
                return s'''
        self.run_test(code, 7, none_loop_break=[int])

    def test_none_loop_continue(self):
        code = '''
            def none_loop_continue(n):
                x = [None if i%2 else 1 for i in range(n)]
                s = 3
                for j in x:
                    if j is None:
                        continue
                    s += 1
                return s'''
        self.run_test(code, 7, none_loop_continue=[int])

    def test_none_loop_break_continue(self):
        code = '''
            def none_loop_break_continue(n):
                x = [None if i%2 else 1 for i in range(n)]
                s = 3
                for j in x:
                    if j is None:
                        s *= 9
                        continue
                    else:
                        s += 1
                        break
                return s'''
        self.run_test(code, 7, none_loop_break_continue=[int])

    def test_none_loop_break_or_ret(self):
        code = '''
            def none_loop_break_or_ret(n):
                x = [None if i%2 else 1 for i in range(n)]
                s = 3
                for j in x:
                    if j is None:
                        break
                    else:
                        return 3
                    s += 1
                return s'''
        self.run_test(code, 7, none_loop_break_or_ret=[int])

    def test_none_loop_continue_or_ret(self):
        code = '''
            def none_loop_continue_or_ret(n):
                x = [None if i%2 else 1 for i in range(n)]
                s = 3
                for j in x:
                    if j is None:
                        continue
                    else:
                        return 3
                    s += 1
                return s'''
        self.run_test(code, 7, none_loop_continue_or_ret=[int])

    def test_none_loop_break_continue_or_ret(self):
        code = '''
            def none_loop_break_continue_or_ret(n):
                x = [None if i%2 else 1 for i in range(n)]
                s = 3
                for j in x:
                    if j is None:
                        s *= 9
                        break
                    elif j < 5:
                        s += 1
                        continue
                    else:
                        return 8
                return s'''
        self.run_test(code, 7, none_loop_break_continue_or_ret=[int])

    def test_none_operators0(self):
        code = '''
            def helper(x):
                if x is not None:
                    x = 3.5
                x += 1
                x *= 5
                x -= 3
                x /= 2
                x //= 2
                return (x + 1,
                        x - 1,
                        x / 3,
                        x // 3,
                        x % 3,
                        x * 3,
                        x ** 3)
            def none_operators0(x):
                if x > 1:
                    y = None
                else:
                    y = 2
                return helper(x), helper(y)'''

        self.run_test(code, 0, none_operators0=[int])

    def test_none_diorcet0(self):
        code = '''
            def none_diorcet0(a):
                x = None if a < 0 else 1
                y = None if a % 2 else 2
                z = -1

                # Doesn't compile
                if x is not None and y is not None:  # Without test on other than none it doesn't work
                    z = 0

                # Doesn't compile
                if x is not None:
                    if y is not None:
                        return 0

                # Doesn't compile
                if x is not None:
                    if y is not None and a != -666:
                        z = 0

                # Compile but wrong results
                if x is not None:
                    if y is not None:
                        z = 0

                # Compile but wrong results (not the same that previous one)
                if x is not None and a != -666 and y is not None:
                    z = 0
                return z'''
        self.run_test(code, 3, none_diorcet0=[int])
        self.run_test(code, 2, none_diorcet0=[int])
        self.run_test(code, -2, none_diorcet0=[int])
        self.run_test(code, -3, none_diorcet0=[int])

    def test_none_diorcet1(self):
        code = '''
            def none_diorcet1(l):
                import numpy as np
                return tuple([None if np.isnan(a) else a for a in l])'''
        self.run_test(code, [3., float('nan')], none_diorcet1=[List[float]])

    def test_none_diorcet2(self):
        code = '''
            def none_diorcet2(headers):
                errors = []
                xxx = None

                def add_error(type, args):
                    errors.append((type, args))

                if "DUMMY_PYTHRAN" in headers:
                    xxx = True

                if xxx is not None:
                    add_error(1, ['AAAAA'])

                if xxx is not None and xxx: # Can't compile
                    add_error(1, ['AAAAA'])

                if "DUMMY_PYTHRAN" in headers: # Without that we have a missing symbol at runtime
                    add_error(2, ['DUMMY_PYTHRAN'])
                return errors'''
        self.run_test(code, {"DUMMY_PYTHRAN":"DUMMY_PYTHRAN"}, none_diorcet2=[Dict[str,str]])

    def test_none_escaping0(self):
        code = '''
            def none_escaping0(i):
                a = 0 
                c = None
                if i % 2 == 0:
                    c = 1
                if c is not None:
                    a = 2 
                    b = 3 
                    if i < 5:
                        print(b + i)
                return a'''
        self.run_test(code, 3, none_escaping0=[int])

    def test_none_escaping1(self):
        code = '''
            def none_escaping1(headers):
                def get_header():
                        return None

                lpix = float(headers.get('a'))

                diameter = None
                if lpix is not None and lpix > 0.0:
                    diameter = 20.0
                return diameter'''
        self.run_test(code, {'a': '10'}, none_escaping1=[Dict[str,str]])


class TestIsInstance(TestEnv):

    def test_isinstance_int0(self):
        self.run_test(
            'def isinstance_int0(x, y): return isinstance(x, int), isinstance(y, int)',
            1, '1',
            isinstance_int0=[int, str])

    def test_isinstance_int1(self):
        code = 'def isinstance_int1a(x):\n if isinstance(x, int): return x  + 1\n else: return  x * 3'
        self.run_test(code, 1, isinstance_int1a=[int])
        code = 'def isinstance_int1b(x):\n if isinstance(x, int): return x  + 1\n else: return  x * 3'
        self.run_test(code, '1', isinstance_int1b=[str])

    def test_isinstance_int2(self):
        code = 'def isinstance_int2(x):\n if isinstance(x, int): return 1\n else: return  "3"'
        self.run_test(code, 1, isinstance_int2=[int])

    def test_isinstance_int3(self):
        code = 'def isinstance_int3(x):\n if isinstance(x, int) or x is None: return 1\n else: return  "3"'
        self.run_test(code, 1, isinstance_int3=[int])

    def test_isinstance_multi0(self):
        code = 'def isinstance_multi0(x):\n if isinstance(x, (int, float)): return 1\n else: return  "3"'
        self.run_test(code, 1, isinstance_multi0=[int])

    def test_isinstance_multi1(self):
        code = 'def isinstance_multi1(x):\n if isinstance(x, (int, float)): return 1\n else: return  "3"'
        self.run_test(code, 1., isinstance_multi1=[float])

    def test_isinstance_multi2(self):
        code = 'def isinstance_multi2(x):\n if isinstance(x, (int, float)): return 1\n else: return  "3"'
        self.run_test(code, "1", isinstance_multi2=[str])

    def test_isinstance_complex0(self):
        code = '''
            import numpy as np

            def conj (x):
                if isinstance(x.dtype, complex):
                    return x.real - 1j*x.imag
                else:
                    return x

            def isinstance_complex0(x):
                return x * conj(x)'''
        self.run_test(code, np.ones(5, dtype=int),
                      isinstance_complex0=[NDArray[int,:]])

    def test_isinstance_complex1(self):
        code = '''
            import numpy as np

            def conj (x):
                if isinstance(x.dtype, complex):
                    return x.real - 1j*x.imag
                else:
                    return x

            def isinstance_complex1(x):
                return x * conj(x)'''
        self.run_test(code, np.ones(5, dtype=complex) * 2j,
                      isinstance_complex1=[NDArray[complex,:]])

    def test_inner_loop_break(self):
        code = '''
import numpy as np
from math import sqrt

def hex_area (spot_radius):
    return sqrt(3)/2 * 3 * spot_radius**2

def inner_loop_break(center, pts, radius, spot_radius, target_area=None):
    spot_area = hex_area (spot_radius)
    pts_within_radius = pts[np.abs (pts - center) <= radius]
    if target_area is None:
        return pts_within_radius
    else:
        area = 0
        pts_list = []
        for pt in pts_within_radius:
            area += spot_area
            if area < target_area:
                pts_list.append (pt)
            else:
                break
        return pts_list
            '''
        self.run_test(code, 1j, np.array([1j]), 1., 1., 1., inner_loop_break=[complex, NDArray[complex, :], float, float, float])

    def test_escape_redef(self):
        code = '''
    def escape_redef(a=None):
        l = []
        if a is None:
            for ii in range(10):
                l.append(ii)
        else:
            a=a

        for ii in range(10):
            l.append(ii)
        return l'''
        self.run_test(code, escape_redef=[])

    def test_nested_static_if0(self):
        code = '''
def nested_static_if0 (
        free_age_vars,
        n_calls
):
    if n_calls is not None:
        n_calls[0] += 1

    cpp_free = free_age_vars[0] and free_age_vars[1]

    QQQQ = 1


    if QQQQ == 0:
        pass
    elif QQQQ == 1:
        if cpp_free:
            if n_calls is not None and n_calls[0] == 1:
                return 4.
        else:
            pass
    elif QQQQ == 2:
        pass

    return 123.'''
        self.run_test(code, [0,1], [0], nested_static_if0=[List[int], List[int]])



#from http://rosettacode.org/wiki/Pick_random_element#Python
#pythran export test()

def test():
    import random
    res = {"foo":0, "bar":0, "baz":0}
    for i in range(500):
        res[random.choice(res.keys())] += 1
    return res



# -*- coding: utf-8 -*-
#
# Pythran documentation build configuration file, created by
# sphinx-quickstart on Wed Feb 19 20:57:04 2014.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import sys
import os
import re

from pythran import __version__

with open("../README.rst") as readme:
    readme_body = readme.read()
    toc = '''

.. toctree::
    :maxdepth: 1


    MANUAL
    EXAMPLES
    CLI
    SUPPORT
    DEVGUIDE
    TUTORIAL
    INTERNAL
    LICENSE
    AUTHORS
    Changelog
'''

    readme_body = readme_body.replace('https://pythran.readthedocs.io', toc)

    with open("index.rst", "w") as index:
        index.write(readme_body)
    del readme_body

with open("../LICENSE") as license:
    with open('LICENSE.rst', 'w') as license_rst:
        license_rst.write("=======\nLICENSE\n=======\n\n")
        license_rst.write(license.read())

with open("../Changelog") as changelog:
    with open('Changelog.rst', 'w') as changelog_rst:
        changelog_rst.write('=========\nChangelog\n=========\n\n')
        changelog_rst.write(changelog.read())

with open("../AUTHORS") as authors:
    with open('AUTHORS.rst', 'w') as authors_rst:
        authors_rst.write(authors.read())

def make_support():
    from pythran import tables

    TITLE = "Supported Modules and Functions"

    DEPTHS = '=*-+:~#.^"`'

    body = []

    body.append(DEPTHS[0]*len(TITLE))
    body.append(TITLE)
    body.append(DEPTHS[0]*len(TITLE))
    body.append("")


    def format_name(name):
        if name.endswith('_') and not name.startswith('_'):
            name = name[:-1]
        return name


    def isiterable(obj):
        return hasattr(obj, '__iter__')


    def dump_entry(entry_name, entry_value, depth):
        if isiterable(entry_value):
            body.append(entry_name)
            body.append(DEPTHS[depth] * len(entry_name))
            body.append("")
            sym_entries, sub_entries = [], []
            for sym in entry_value:
                w = sub_entries if isiterable(entry_value[sym]) else sym_entries
                w.append(sym)
            for k in sorted(sym_entries):
                dump_entry(format_name(k), entry_value[k], depth + 1)
            body.append("")
            for k in sorted(sub_entries):
                dump_entry(format_name(k), entry_value[k], depth + 1)
                body.append("")
        else:
            body.append(entry_name)

    for MODULE in sorted(tables.MODULES):
        if MODULE != '__dispatch__':
            dump_entry(format_name(MODULE), tables.MODULES[MODULE], 1)

    return "\n".join(body)

with open('SUPPORT.rst', 'w') as support:
    support.write(make_support())


# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#sys.path.insert(0, os.path.abspath('.'))

# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = ['nbsphinx',]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The encoding of source files.
#source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'Pythran'
copyright = u'2014, Serge Guelton, Pierrick Brunet et al.'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = __version__
# The full version, including alpha/beta/rc tags.
release = version

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
#today = ''
# Else, today_fmt is used as the format for a strftime call.
#today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ['**.ipynb_checkpoints']

# The reST default role (used for this markup: `text`) to use for all
# documents.
#default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
#add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
#add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
#show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# A list of ignored prefixes for module index sorting.
#modindex_common_prefix = []

# If true, keep warnings as "system message" paragraphs in the built documents.
#keep_warnings = False


# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
import guzzle_sphinx_theme

html_theme_path = guzzle_sphinx_theme.html_theme_path()
html_theme = 'guzzle_sphinx_theme'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
html_theme_options = {
    # Set the name of the project to appear in the sidebar
    "project_nav_name": "Project Name",
}

# Add any paths that contain custom themes here, relative to this directory.
#html_theme_path = []

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
#html_title = None

# A shorter title for the navigation bar.  Default is the same as html_title.
#html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
# html_logo = 'pythran.png'

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
#html_favicon = None

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
#html_static_path = ['_static']

# Add any extra paths that contain custom files (such as robots.txt or
# .htaccess) here, relative to this directory. These files are copied
# directly to the root of the documentation.
#html_extra_path = []

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
#html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
#html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
html_sidebars = {'**': ['globaltoc.html']}

# Additional templates that should be rendered to pages, maps page names to
# template names.
#html_additional_pages = {}

# If false, no module index is generated.
html_domain_indices = False

# If false, no index is generated.
html_use_index = False

# If true, the index is split into individual pages for each letter.
#html_split_index = False

# If true, links to the reST sources are added to the pages.
html_show_sourcelink = False

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
#html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
#html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
#html_file_suffix = None

# Output file base name for HTML help builder.
htmlhelp_basename = 'Pythrandoc'




#from http://stackoverflow.com/questions/18345202/functional-vs-imperative-style-in-python
#pythran export scrabble_fun_score(str, str: int dict)
#pythran export scrabble_imp_score(str, str: int dict)
#runas scrabble_fun_score('tralala', {'t': 1, 'r': 2, 'a': 3, 'l': 4})
#runas scrabble_fun_score('tralala', {'t': 1, 'r': 2, 'a': 3, 'l': 4})
#bench import string; import random; a = "".join([random.choice(string.letters) for i in range(12000000)]); v = dict(zip(string.letters, range(1000))); scrabble_fun_score(a, v)

def scrabble_fun_score(word, scoretable):
    return sum([scoretable.get(x, 0) for x in word])


def scrabble_imp_score(word, scoretable):
    score = 0
    for letter in word:
        if letter in scoretable:
            score += scoretable[letter]
    return score



#from http://rosettacode.org/wiki/Power_set#Python
#pythran export p(int list)
#pythran export list_powerset(int list)
#pythran export list_powerset2(int list)
#runas p([1,2,3])
#runas list_powerset([1,2,3])
#runas list_powerset2([1,2,3])

from functools import reduce

def list_powerset(lst):
    # the power set of the empty set has one element, the empty set
    result = [[]]
    for x in lst:
        # for every additional element in our set
        # the power set consists of the subsets that don't
        # contain this element (just take the previous power set)
        # plus the subsets that do contain the element (use list
        # comprehension to add [x] onto everything in the
        # previous power set)
        result.extend([subset + [x] for subset in result])
    return result

# the above function in one statement
def list_powerset2(lst):
    return reduce(lambda result, x: result + [subset + [x] for subset in result],
            lst, [[]])

def p(l):
    if not l: return [[]]
    return p(l[1:]) + [[l[0]] + x for x in p(l[1:])]




import numpy as np
def aaa(a):
    grid = np.empty((a, ))
    # omp parallel for private(i, j)
    for i in range(a):
        if i % 2:
            j = i * i
        else:
            j = i * i *3
        grid[i] = j

def pythran_private():
    from random import randint
    return aaa(randint(5,5))



#pythran export match_arrays (float32[], float32[])
#pythran export match_arrays (int16[], int16[])
#runas import numpy as np; x = np.array([1,2,3], dtype=np.int16); y = np.array([3,4,5], dtype=np.int16); match_arrays(x, y)
import numpy as np
def match_arrays(a, b):    # pragma: no cover
    """
    Finds all indexes into a and b such that a[i] = b[j]. The outputs are sorted
    in lexographical order.

    Parameters
    ----------
    a, b : np.ndarray
        The input 1-D arrays to match. If matching of multiple fields is
        needed, use np.recarrays. These two arrays must be sorted.

    Returns
    -------
    a_idx, b_idx : np.ndarray
        The output indices of every possible pair of matching elements.
    """
    if len(a) == 0 or len(b) == 0:
        return np.empty(0, dtype=np.uintp), np.empty(0, dtype=np.uintp)

    a_ind, b_ind = [], []
    nb = len(b)
    ib = 0
    match = 0

    for ia, j in enumerate(a):
        if j == b[match]:
            ib = match

        while ib < nb and j >= b[ib]:
            if j == b[ib]:
                a_ind.append(ia)
                b_ind.append(ib)

                if b[match] < b[ib]:
                    match = ib

            ib += 1

    return np.array(a_ind, dtype=np.uintp), np.array(b_ind, dtype=np.uintp)



import numpy as np

#runas import numpy as np; r = np.arange(0.,10., .01); rosen(r)
#bench import numpy as np; r = np.arange(50000000); rosen(r)
#pythran export rosen(int[])
#pythran export rosen(float[])

def rosen(x):
    t0 = 100 * (x[1:] - x[:-1] ** 2) ** 2
    t1 = (1 - x[:-1]) ** 2
    return np.sum(t0 + t1)



def omp_parallel_private():
    sum = 0
    num_threads = 0

    if 'omp parallel':
        sum1 = 7
        'omp for'
        for i in range(1, 1000):
            sum1 += i

        if 'omp critical':
            sum += sum1
            num_threads += 1

    known_sum = (999 * 1000) / 2 + 7 * num_threads

    return known_sum == sum



#runas solve(500)
#pythran export solve(int)
def solve(nfact):

    prime_list = [2, 3, 5, 7, 11, 13, 17, 19, 23]   # Ensure that this is initialised with at least 1 prime
    prime_dict = dict.fromkeys(prime_list, 1)

    def _isprime(n):
        ''' Raw check to see if n is prime. Assumes that prime_list is already populated '''
        isprime = n >= 2 and 1 or 0
        for prime in prime_list:                    # Check for factors with all primes
            if prime * prime > n: break             # ... up to sqrt(n)
            if not n % prime:
                isprime = 0
                break
        if isprime: prime_dict[n] = 1               # Maintain a dictionary for fast lookup
        return isprime

    def prime(x):
        ''' Returns the xth prime '''

        lastn = prime_list[-1]
        while len(prime_list) <= x:                 # Keep working until we've got the xth prime
            lastn = lastn + 1                       # Check the next number
            if _isprime(lastn):
                prime_list.append(lastn)            # Maintain a list for sequential access
        return prime_list[x]

    def num_factors(n):
        ''' Returns the number of factors of n, including 1 and n '''
        div = 1
        x = 0
        while n > 1:
            c = 1
            while not n % prime(x):
                c = c + 1
                n = n // prime(x)
            x = x + 1
            div = div * c
        return div

    for i in range(1, 1000000000):
        n = i * (i+1) // 2
        if num_factors(n) > nfact:
            return n
            break



#from http://rosettacode.org/wiki/Pascal%27s_triangle#Python
#pythran export pascal(int)
#pythran export pascal_(int)
#runas pascal(10)
#runas pascal_(10)

def pascal(n):
    """Prints out n rows of Pascal's triangle.
    It returns False for failure and True for success."""
    row = [1]
    k = [0]
    for x in range(max(n,0)):
        print(row)
        row=[l+r for l,r in zip(row+k,k+row)]
    return n>=1

def scan(op, seq, it):
    a = []
    result = it
    a.append(it)
    for x in seq:
        result = op(result, x)
        a.append(result)
    return a

def pascal_(n):
    def nextrow(row, x):
        return [l+r for l,r in zip(row+[0,],[0,]+row)]
    return scan(nextrow, range(n-1), [1,])



def omp_sections_nowait():
    import omp
    from time import sleep
    result = False
    count = 0

    if 'omp parallel':
        in_parallel = omp.in_parallel()
        rank = omp.get_thread_num()

        if 'omp sections nowait':
            if 'omp section':
                sleep(0.01)
                count = 1
                'omp flush(count)'
            if 'omp section':
                pass

        if 'omp sections':
            if 'omp section':
                pass
            if 'omp section':
                if count == 0:
                    result = True
    return result or not in_parallel



#pythran export CDOTC(int, complex list, int, complex list, int)
#runas CDOTC(2,[complex(1,2),complex(2,3),complex(3,4),complex(5,6)],2,[complex(3,4),complex(1,2),complex(2,3),complex(5,6)],3)
#bench sz = 20000000; in1 = map(complex, range(sz), range(sz)); in2 = map(complex, range(sz), range(sz));CDOTC(sz / 6,in1,2,in2,3)
def CDOTC(N,CX,INCX,CY,INCY):
#     .. Scalar Arguments ..
#      INTEGER INCX,INCY,N
#     ..
#     .. Array Arguments ..
#      COMPLEX CX(#),CY(#)
#     ..
#
#  Purpose
#  =======
#
#     forms the dot product of two vectors, conjugating the first
#     vector.
#
#  Further Details
#  ===============
#
#     jack dongarra, linpack,  3/11/78.
#     modified 12/3/93, array(1) declarations changed to array(#)
#
#  =====================================================================
#
    CTEMP = complex(0.0,0.0)
    CDOTC = complex(0.0,0.0)
    if (N <= 0):
        return
    if (INCX == 1 and  INCY == 1):

#
#        code for both increments equal to 1
#
        for I in range(N):
            CTEMP = CTEMP + (CX[I].conjugate())*CY[I]
    else:
#
#        code for unequal increments or equal increments
#          not equal to 1
#
        IX = 0
        IY = 0
        if (INCX < 0):
            IX = (-N+1)*INCX
        if (INCY < 0):
            IY = (-N+1)*INCY
        for I in range(N):
            CTEMP = CTEMP + (CX[IX].conjugate())*CY[IY]
            IX = IX + INCX
            IY = IY + INCY
    return CTEMP




# pythran export solve(int, int)
# runas solve(1, 10)


def solve(start, end):
    """
    2520 is the smallest number that can be divided by each of the numbers from
    1 to 10 without any remainder.

    What is the smallest number that is evenly divisible by all of the numbers
    from 1 to 20?
    """
    def gcd(a, b):
        return b and gcd(b, a % b) or a

    def lcm(a, b):
        return a * b / gcd(a, b)

    n = 1
    for i in range(start, end + 1):
        n = lcm(n, i)

    return n



#runas solve()
#pythran export solve()
def solve():
    '''
    It can be seen that the number, 125874, and its double, 251748, contain exactly the same digits, but in a different order.
    
    Find the smallest positive integer, x, such that 2x, 3x, 4x, 5x, and 6x, contain the same digits.
    '''

    def multiples_have_same_digits(n):
        digit_keys = dict.fromkeys(list(str(n)))
        for x in range(2, 4):
            for d in list(str(x * n)):
                if d not in digit_keys: return False
        return True

    n = 0
    while True:
        n = n + 9                           # n must be a multiple of 9 for this to happen
        if multiples_have_same_digits(n):
            return n
            break




#runas solve()
#pythran export solve()
def solve():
    '''
    Take the number 192 and multiply it by each of 1, 2, and 3:

    192 x 1 = 192
    192 x 2 = 384
    192 x 3 = 576
    By concatenating each product we get the 1 to 9 pandigital, 192384576. We will call 192384576 the concatenated product of 192 and (1,2,3)

    The same can be achieved by starting with 9 and multiplying by 1, 2, 3, 4, and 5, giving the pandigital, 918273645, which is the concatenated product of 9 and (1,2,3,4,5).

    What is the largest 1 to 9 pandigital 9-digit number that can be formed as the concatenated product of an integer with (1,2, ... , n) where n > 1?
    '''

    def get_pandigital(n):
        pandigital = ''
        for x in range(1, 10):
            pandigital += str(x * n)
            if len(pandigital) >= 9: break
        if len(pandigital) == 9 and sorted(dict.fromkeys(list(pandigital)).keys()) == list("123456789"): return pandigital
        else: return ''

    max = ''
    for n in range(1, 10000):
        p = get_pandigital(n)
        if p and p > max: max = p

    return max




"""
This file demonstrates a filterbank correlation loop.
"""

#pythran export fbcorr(float[][][][], float[][][][], float[][][][])
#bench import numpy; in_ = numpy.arange(10*20*30*7.).reshape(10,20,30,7); filter = numpy.arange(2*3*4*7.).reshape(2,3,4,7); out = numpy.empty((10,2,18,27), dtype=numpy.float); fbcorr(in_, filter, out)
def fbcorr(imgs, filters, output):
    n_imgs, n_rows, n_cols, n_channels = imgs.shape
    n_filters, height, width, n_ch2 = filters.shape

    "omp parallel for"
    for ii in range(n_imgs):
        for rr in range(n_rows - height + 1):
            for cc in range(n_cols - width + 1):
                for hh in range(height):
                    for ww in range(width):
                        for jj in range(n_channels):
                            for ff in range(n_filters):
                                imgval = imgs[ii, rr + hh, cc + ww, jj]
                                filterval = filters[ff, hh, ww, jj]
                                output[ii, ff, rr, cc] += imgval * filterval




#from: http://stackoverflow.com/questions/4651683/numpy-grouping-using-itertools-groupby-performance
#runas import numpy as np ; values = np.array([1,3,5,2,4],dtype="u4") ; values.sort() ; grouping(values)
#bench import numpy as np ; N = 350000 ; values = np.array(np.random.randint(0,3298,size=N),dtype="u4") ; values.sort() ; grouping(values)

#pythran export grouping(uint32 [])

def grouping(values):
    import numpy as np
    diff = np.concatenate(([1], np.diff(values)))
    idx = np.concatenate((np.where(diff)[0], [len(values)]))
    return values[idx[:-1]], np.diff(idx)



# Reference: http://continuum.io/blog/the-python-and-the-complied-python
#pythran export diffusePurePython(float [][], float [][], int)
#runas import numpy as np;lx,ly=(2**7,2**7);u=np.zeros([lx,ly],dtype=np.double);u[int(lx/2),int(ly/2)]=1000.0;tempU=np.zeros([lx,ly],dtype=np.double);diffusePurePython(u,tempU,500)
#bench import numpy as np;lx,ly=(2**6,2**6);u=np.zeros([lx,ly],dtype=np.double);u[int(lx/2),int(ly/2)]=1000.0;tempU=np.zeros([lx,ly],dtype=np.double);diffusePurePython(u,tempU,55)

import numpy as np


def diffusePurePython(u, tempU, iterNum):
    """
    Apply nested iteration for the Forward-Euler Approximation
    """
    mu = .1
    row = u.shape[0]
    col = u.shape[1]

    for n in range(iterNum):
        for i in range(1, row - 1):
            for j in range(1, col - 1):
                tempU[i, j] = u[i, j] + mu * (
                    u[i + 1, j] - 2 * u[i, j] + u[i - 1, j] +
                    u[i, j + 1] - 2 * u[i, j] + u[i, j - 1])
        for i in range(1, row - 1):
            for j in range(1, col - 1):
                u[i, j] = tempU[i, j]
                tempU[i, j] = 0.0



#runas solve()
#unittest.skip recursive generator
#pythran export solve()

''' From O'Reilly's Python Cookbook '''

def _combinators(_handle, items, n):
    if n==0:
        yield []
        return
    for i, item in enumerate(items):
        this_one = [ item ]
        for cc in _combinators(_handle, _handle(items, i), n-1):
            yield this_one + cc

def combinations(items, n):
    ''' take n distinct items, order matters '''
    def skipIthItem(items, i):
        return items[:i] + items[i+1:]
    return _combinators(skipIthItem, items, n)

def uniqueCombinations(items, n):
    ''' take n distinct items, order is irrelevant '''
    def afterIthItem(items, i):
        return items[i+1:]
    return _combinators(afterIthItem, items, n)

def selections(items, n):
    ''' take n (not necessarily distinct) items, order matters '''
    def keepAllItems(items, i):
        return items
    return _combinators(keepAllItems, items, n)

def permutations(items):
    ''' take all items, order matters '''
    return combinations(items, len(items))

def solve():
    '''
    The product 7254 is unusual, as the identity, 39 x 186 = 7254, containing multiplicand, multiplier, and product is 1 through 9 pandigital.

    Find the sum of all products whose multiplicand/multiplier/product identity can be written as a 1 through 9 pandigital.

    HINT: Some products can be obtained in more than one way so be sure to only include it once in your sum.
    '''

    ''' From O'Reilly's Python Cookbook '''


    def num(l):
        s = 0
        for n in l: s = s * 10 + n
        return s

    product = {}
    for perm in permutations(range(1,10)):
        for cross in range(1,4):            # Number can't be more than 4 digits
            for eq in range(cross+1, 6):    # Result can't be less than 4 digits
                a = num(perm[0:cross])
                b = num(perm[cross:eq])
                c = num(perm[eq:9])
                if a * b == c: product[c] = 1

    return sum(p for p in product)




from pythran.tests import TestEnv

class TestBase(TestEnv):

    def test_rec0(self):
        self.run_test("""
def test_rec0(n):
  z = 1
  if n > 1:
    z = n * test_rec0(n-1)
  return z""", 5, test_rec0=[int])

    def test_rec1(self):
        self.run_test("""
def test_rec1(n):
  z = 1
  while n > 1:
    z = n * test_rec1(n-1)
    n -= 1
  return z""", 5, test_rec1=[int])



import omp
from time import sleep


def omp_for_schedule_guided():
    tids = list(range(1001))
    maxiter = 0
    result = True
    notout = True
    if 'omp parallel num_threads(4)':
        in_parallel = omp.in_parallel()
        if 'omp single':
            threads = omp.get_num_threads()

    if threads<2:
        print("This test only works with at least two threads")
        result = False

    if 'omp parallel shared(tids, maxiter) num_threads(4)':
        tid = omp.get_num_threads()
        'omp for nowait schedule(guided)'
        for j in range(1000):
            count = 0
            'omp flush(maxiter)'
            if j > maxiter:
                if 'omp critical':
                    maxiter = j
            'omp flush(notout, maxiter)'
            while notout and count < 0.0005 and maxiter == j:
                'omp flush(notout, maxiter)'
                sleep(0.0001)
                count += 0.0001
            tids[j] = tid

        notout = False
        'omp flush(maxiter, notout)'

    last_threadnr = tids[0]
    global_chunknr = 0
    local_chunknr = [0 for i in range(10)]
    openwork = 1000;
    tids[1000] = -1

    for i in range(1,1001):
        if last_threadnr == tids[i]:
            pass
        else:
            global_chunknr += 1
            local_chunknr[last_threadnr] += 1
            last_threadnr = tids[i]

    chuncksize = list(range(global_chunknr))

    global_chunknr = 0
    determined_chunksize = 1
    last_threadnr = tids[0]

    for i in range(1,1001):
        if last_threadnr == tids[i]:
            determined_chunksize += 1
        else:
            chuncksize[global_chunknr] = determined_chunksize
            global_chunknr += 1
            local_chunknr[last_threadnr] += 1
            last_threadnr = tids[i]
            determined_chunksize = 1

    expected_chunk_size = openwork / threads
    c = chuncksize[0] / expected_chunk_size

    for i in range(global_chunknr):
        if expected_chunk_size > 1:
            expected_chunk_size = c * openwork / threads
        if abs(chuncksize[i] - expected_chunk_size) >= 2:
            result = False

        openwork -= chuncksize[i]
    return result or not in_parallel



#from http://rosettacode.org/wiki/Permutations/Rank_of_a_permutation#Python
#pythran export test()
#runas test()
from math import factorial as fact
from random import randrange

def identity_perm(n):
    return list(range(n))

def unranker1(n, r, pi):
    while n > 0:
        n1, (rdivn, rmodn) = n-1, divmod(r, n)
        pi[n1], pi[rmodn] = pi[rmodn], pi[n1]
        n = n1
        r = rdivn
    return pi

def init_pi1(n, pi):
    pi1 = [-1] * n
    for i in range(n):
        pi1[pi[i]] = i
    return pi1

def ranker1(n, pi, pi1):
    if n == 1:
        return 0
    n1 = n-1
    s = pi[n1]
    pi[n1], pi[pi1[n1]] = pi[pi1[n1]], pi[n1]
    pi1[s], pi1[n1] = pi1[n1], pi1[s]
    return s + n * ranker1(n1, pi, pi1)

def unranker2(n, r, pi):
    while n > 0:
        n1 = n-1
        s, rmodf = divmod(r, fact(n1))
        pi[n1], pi[s] = pi[s], pi[n1]
        n = n1
        r = rmodf
    return pi

def ranker2(n, pi, pi1):
    if n == 1:
        return 0
    n1 = n-1
    s = pi[n1]
    pi[n1], pi[pi1[n1]] = pi[pi1[n1]], pi[n1]
    pi1[s], pi1[n1] = pi1[n1], pi1[s]
    return s * fact(n1) + ranker2(n1, pi, pi1)

def get_random_ranks(permsize, samplesize):
    perms = fact(permsize)
    ranks = set()
    while len(ranks) < samplesize:
        ranks |= set( randrange(perms)
                      for r in range(samplesize - len(ranks)) )
    return ranks

def test1(comment, unranker, ranker):
    n, samplesize, n2 = 3, 4, 12
    print(comment)
    perms = []
    for r in range(fact(n)):
        pi = identity_perm(n)
        perm = unranker(n, r, pi)
        perms.append((r, perm))
    for r, pi in perms:
        pi1 = init_pi1(n, pi)
        print('  From rank %s to %s back to %s' % (r, pi, ranker(n, pi[:], pi1)))
    print('\n  %s random individual samples of %s items:' % (samplesize, n2))
    for r in get_random_ranks(n2, samplesize):
        pi = identity_perm(n2)
        print('    ' + ' '.join('%s' % i for i in unranker(n2, r, pi)))
    print('')

def test2(comment, unranker):
    samplesize, n2 = 4, 10
    print(comment)
    print('  %s random individual samples of %s items:' % (samplesize, n2))
    txt = ''
    for r in get_random_ranks(n2, samplesize):
        pi = identity_perm(n2)
        txt += '\n' + ''.join(str(unranker(n2, r, pi)))
    print(txt, '')

def test():
    test1('First ordering:', unranker1, ranker1)
    test1('Second ordering:', unranker2, ranker2)
    test2('First ordering, large number of perms:', unranker1)



#runas solve(100)
#pythran export solve(int)
def solve(max):
    '''
    The sum of the squares of the first ten natural numbers is,
    1^2 + 2^2 + ... + 10^2 = 385
    The square of the sum of the first ten natural numbers is,
    (1 + 2 + ... + 10)^2 = 552 = 3025
    Hence the difference between the sum of the squares of the first ten natural numbers and the square of the sum is 3025 - 385 = 2640.
    
    Find the difference between the sum of the squares of the first one hundred natural numbers and the square of the sum.
    '''

    r = range(1, max + 1)
    a = sum(r)
    return a * a - sum(i*i for i in r)



# http://code.activestate.com/recipes/577263-numerical-integration-using-monte-carlo-method/
# Numerical Integration using Monte Carlo method
# FB - 201006137
#pythran export montecarlo_integration(float, float, int, float list, int)
#runas montecarlo_integration(1.,10.,100,[x/100. for x in range(100)],100)
#bench montecarlo_integration(1.,10.,650000,[x/100. for x in range(100)],100)
import math


def montecarlo_integration(xmin, xmax, numSteps,rand,randsize):
    # define any function here!
    def f(x):
        return math.sin(x)

    # find ymin-ymax
    ymin = f(xmin)
    ymax = ymin
    for i in range(numSteps):
        x = xmin + (xmax - xmin) * float(i) / numSteps
        y = f(x)
        if y < ymin: ymin = y
        if y > ymax: ymax = y

    # Monte Carlo
    rectArea = (xmax - xmin) * (ymax - ymin)
    numPoints = numSteps # bigger the better but slower!
    ctr = 0
    for j in range(numPoints):
        x = xmin + (xmax - xmin) * rand[j%randsize]
        y = ymin + (ymax - ymin) * rand[j%randsize]
        if math.fabs(y) <= math.fabs(f(x)):
            if f(x) > 0 and y > 0 and y <= f(x):
                ctr += 1 # area over x-axis is positive
            if f(x) < 0 and y < 0 and y >= f(x):
                ctr -= 1 # area under x-axis is negative

    fnArea = rectArea * float(ctr) / numPoints
    return fnArea



def omp_for_ordered():
    sum = 0
    is_larger = 1
    last_i = 0
    if 'omp parallel':
        my_is_larger = 1
        'omp for schedule(static,1) ordered'
        for i in range(1,100):
            if 'omp ordered':
                my_is_larger &= i > last_i
                last_i = i
                sum += i
        'omp critical'
        is_larger &= my_is_larger
    known_sum = (99 * 100) / 2
    return known_sum == sum and is_larger



#
# Copyright (C)  Tyler Reddy, Richard Gowers, and Max Linke, 2016
#
# Distributed under the same BSD license as Scipy.
#
# adapted from scipy's cython version
import numpy as np
import numpy.random as random

#pythran export directed_hausdorff(float64[:,:], float64[:,:], int)
#pythran export directed_hausdorff_noshuffle(float64[:,:], float64[:,:])
#runas import numpy as np; x = np.arange((100 * 100.)).reshape(100,-1); y =  np.ones((100,100)) * 3; directed_hausdorff_noshuffle(x, y)

def directed_hausdorff(ar1, ar2, seed=0):

    N1, data_dims = ar1.shape
    N2 = ar2.shape[0]
    i_store = j_store = i_ret = j_ret = 0

    # shuffling the points in each array generally increases the likelihood of
    # an advantageous break in the inner search loop and never decreases the
    # performance of the algorithm
    random.seed(seed)
    resort1 = np.arange(N1)
    resort2 = np.arange(N2)
    random.shuffle(resort1)
    random.shuffle(resort2)
    ar1 = np.asarray(ar1)[resort1]
    ar2 = np.asarray(ar2)[resort2]

    cmax = 0
    for i in range(N1):
        cmin = np.inf
        for j in range(N2):
            d = np.sum((ar1[i] - ar2[j]) ** 2)
            # faster performance with square of distance
            # avoid sqrt until very end
            if d < cmax: # break out of `for j` loop
                break

            if d < cmin: # always true on first iteration of for-j loop
                cmin = d
                i_store = i
                j_store = j
        else:

            # always true on first iteration of for-j loop, after that only
            # if d >= cmax
            if cmin != np.inf and cmin > cmax:
                cmax = cmin
                i_ret = i_store
                j_ret = j_store

    return np.sqrt(cmax), resort1[i_ret], resort2[j_ret]

def directed_hausdorff_noshuffle(ar1, ar2, seed=0):

    N1, data_dims = ar1.shape
    N2 = ar2.shape[0]
    i_store = j_store = i_ret = j_ret = 0

    resort1 = np.arange(N1)
    resort2 = np.arange(N2)
    ar1 = np.asarray(ar1)[resort1]
    ar2 = np.asarray(ar2)[resort2]

    cmax = 0
    for i in range(N1):
        cmin = np.inf
        for j in range(N2):
            d = np.sum((ar1[i] - ar2[j]) ** 2)
            # faster performance with square of distance
            # avoid sqrt until very end
            if d < cmax: # break out of `for j` loop
                break

            if d < cmin: # always true on first iteration of for-j loop
                cmin = d
                i_store = i
                j_store = j
        else:

            # always true on first iteration of for-j loop, after that only
            # if d >= cmax
            if cmin != np.inf and cmin > cmax:
                cmax = cmin
                i_ret = i_store
                j_ret = j_store

    return np.sqrt(cmax), resort1[i_ret], resort2[j_ret]




# --- Python / Numpy imports -------------------------------------------------
import numpy as np
from time import time
#pythran export compute_julia(float, float, int, float?, float?, int?)

def kernel(zr, zi, cr, ci, lim, cutoff):
    ''' Computes the number of iterations `n` such that 
        |z_n| > `lim`, where `z_n = z_{n-1}**2 + c`.
    '''
    count = 0
    while ((zr*zr + zi*zi) < (lim*lim)) and count < cutoff:
        zr, zi = zr * zr - zi * zi + cr, 2 * zr * zi + ci
        count += 1
    return count

def compute_julia(cr, ci, N, bound=1.5, lim=1000., cutoff=1e6):
    ''' Pure Python calculation of the Julia set for a given `c`.  No NumPy
        array operations are used.
    '''
    julia = np.empty((N, N), np.uint32)
    grid_x = np.linspace(-bound, bound, N)
    t0 = time()
    "omp parallel for default(none) shared(grid_x, cr, ci, lim, cutoff, julia)"
    for i, x in enumerate(grid_x):
        for j, y in enumerate(grid_x):
            julia[i,j] = kernel(x, y, cr, ci, lim, cutoff)
    return julia, time() - t0



import numpy as np
#runas import numpy as np; np.random.seed(0); s=np.array([1+1j, -1+1j, -1-1j, 1-1j]); e=np.random.choice(s, (1, 100)); ee=e/np.sqrt(np.mean(abs(e)**2)); eee=ee+0.02*(np.random.randn(*ee.shape)+1j*np.random.randn(*ee.shape)); r=(np.sqrt(2)/2+0j);wx=np.zeros((1,10), dtype=eee.dtype); wx[:,4]=1; train_eq(eee.astype(np.complex64), eee.shape[1]-15, 1, np.float32(1e-3), wx.astype(np.complex64), (np.complex64(r),s.astype(np.complex64)), False)

#runas import numpy as np; np.random.seed(0); s=np.array([1+1j, -1+1j, -1-1j, 1-1j]); e=np.random.choice(s, (1, 100)); ee=e/np.sqrt(np.mean(abs(e)**2)); eee=ee+0.02*(np.random.randn(*ee.shape)+1j*np.random.randn(*ee.shape)); r=(np.sqrt(2)/2+0j);wx=np.zeros((1,10), dtype=eee.dtype); wx[:,4]=1; train_eq2(eee.astype(np.complex64), eee.shape[1]-15, 1, np.float32(1e-3), wx.astype(np.complex64), (np.complex64(r),s.astype(np.complex64)), False)

#pythran export train_eq(complex64[][], int, int, float32,
        # complex64[][],
        # (complex64, complex64[]), bool)
#pythran export train_eq2(complex64[][], int, int, float32,
        # complex64[][],
        # (complex64, complex64[]), bool)
def train_eq(E, TrSyms, os, mu, wx,  errfctprs, adapt):
    Ntaps = wx.shape[1]
    pols = wx.shape[0]
    R, symbs = errfctprs
    err = np.zeros(TrSyms, dtype=E.dtype)
    for i in range(TrSyms):
        X = E[:, i * os:i * os + Ntaps]
        Xest = np.sum(np.conj(wx) * X)
        err[i] = (R.real - abs(Xest)**2)*Xest
        wx += mu*np.conj(err[i])*X
        if adapt and i>0:
            if err[i].real*err[i-1].real > 0 and err[i].imag*err[i-1].imag > 0:
                mu = mu
            else:
                mu = np.float32(mu/(1+mu*(err[i].real*err[i].real + err[i].imag*err[i].imag)))
    return err, wx, mu

def train_eq2(E, TrSyms, os, mu, wx,  errfctprs, adapt):
    Ntaps = wx.shape[1]
    pols = wx.shape[0]
    R, symbs = errfctprs
    err = np.zeros(TrSyms, dtype=E.dtype)
    for i in range(TrSyms):
        X = E[:, i * os:i * os + Ntaps]
        Xest = np.sum(np.conj(wx) * X)
        err[i] = (R.real - abs(Xest)**2)*Xest
        wx += mu*np.conj(err[i])*X
        if adapt and i>0:
            if err[i].real*err[i-1].real > 0 and err[i].imag*err[i-1].imag > 0:
                mu = mu
            else:
                mu = np.float32(mu/(1+mu*(err[i].real*err[i].real + err[i].imag*err[i].imag)))
    return err, wx, mu

def cma_error(Xest, R, symbs):
    return (R.real - abs(Xest)**2)*Xest

def mcma_error(Xest, R, symbs):
    return symbs.dtype.type((R.real - Xest.real**2)*Xest.real + (R.imag - Xest.imag**2)*Xest.imag*1j)



#from http://www.parakeetpython.com/
#pythran export smoothing(float[], float)
#runas import numpy as np ; a = np.arange(0,1,10e-3) ; smoothing(a, .4)
#bench import numpy as np ; a = np.arange(0,1,1.5e-6) ;smoothing(a, .4)

def smoothing(x, alpha):
  """
  Exponential smoothing of a time series
  For x = 10**6 floats
  - Python runtime: 9 seconds
  - Parakeet runtime: .01 seconds
  """
  s = x.copy()
  for i in range(1, len(x)):
    s[i] = alpha * x[i] + (1 - alpha) * s[i-1]
  return s



import numpy as np

from numpy import trace, concatenate, dot
from numpy.random import randn
from numpy.linalg import matrix_power

# pythran export rand_mat_stat(int)

def rand_mat_stat(t):
    n = 5
    v = np.empty(t)
    w = np.empty(t)
    for i in range(t):
        a = randn(n, n)
        b = randn(n, n)
        c = randn(n, n)
        d = randn(n, n)
        P = concatenate((a, b, c, d), axis=1)
        Q = concatenate((concatenate((a, b), axis=1),
                         concatenate((c, d), axis=1)), axis=0)
#        v[i] = trace(matrix_power(P.T @ P, 4))
#        w[i] = trace(matrix_power(Q.T @ Q, 4))
        v[i] = trace(matrix_power(dot(P.T, P), 4))
        w[i] = trace(matrix_power(dot(Q.T, Q), 4))
    return np.std(v)/np.mean(v), np.std(w)/np.mean(w)



from numpy.distutils.core import setup
from numpy.distutils.command.build_ext import build_ext as npy_build_ext
from pythran.dist import PythranExtension, PythranBuildExt

module1 = PythranExtension('demo3.a', sources = ['demo3/a.py'])

setup(name = 'demo3',
      version = '1.0',
      description = 'This is a demo package',
      cmdclass={"build_ext": PythranBuildExt[npy_build_ext]},
      ext_modules = [module1])



#runas solve(1, 1000)
#pythran export solve(int, int)
#FIXME unittest.skip conflicting name for end
def solve(start, end):
    '''
    How many letters would be needed to write all the numbers in words from 1 to 1000?
    '''

    words = [
        (   1,  'one'      , ''     ),
        (   2,  'two'      , ''     ),
        (   3,  'three'    , ''     ),
        (   4,  'four'     , ''     ),
        (   5,  'five'     , ''     ),
        (   6,  'six'      , ''     ),
        (   7,  'seven'    , ''     ),
        (   8,  'eight'    , ''     ),
        (   9,  'nine'     , ''     ),
        (  10,  'ten'      , ''     ),
        (  11,  'eleven'   , ''     ),
        (  12,  'twelve'   , ''     ),
        (  13,  'thirteen' , ''     ),
        (  14,  'fourteen' , ''     ),
        (  15,  'fifteen'  , ''     ),
        (  16,  'sixteen'  , ''     ),
        (  17,  'seventeen', ''     ),
        (  18,  'eighteen' , ''     ),
        (  19,  'nineteen' , ''     ),
        (  20,  'twenty'   , ''     ),
        (  30,  'thirty'   , ''     ),
        (  40,  'forty'    , ''     ),
        (  50,  'fifty'    , ''     ),
        (  60,  'sixty'    , ''     ),
        (  70,  'seventy'  , ''     ),
        (  80,  'eighty'   , ''     ),
        (  90,  'ninety'   , ''     ),
        ( 100,  'hundred'  , 'and'  ),
        (1000,  'thousand' , 'and'  ),
    ]
    words.reverse()

    def spell(n, words):
        word = []
        while n > 0:
            for num in words:
                if num[0] <= n:
                    div = n / num[0]
                    n = n % num[0]
                    if num[2]: word.append(' '.join(spell(div, words)))
                    word.append(num[1])
                    if num[2] and n: word.append(num[2])
                    break
        return word

    return sum(len(word) for n in range(start, end + 1) for word in spell(n, words))




#from http://rosettacode.org/wiki/Greatest_subsequential_sum#Python
#pythran export maxsum(int list)
#pythran export maxsumseq(int list)
#pythran export maxsumit(int list)
#runas maxsum([0, 1, 0])
#runas maxsumseq([-1, 2, -1, 3, -1])
#runas maxsumit([-1, 1, 2, -5, -6])

def maxsum(sequence):
    """Return maximum sum."""
    maxsofar, maxendinghere = 0, 0
    for x in sequence:
        # invariant: ``maxendinghere`` and ``maxsofar`` are accurate for ``x[0..i-1]``          
        maxendinghere = max(maxendinghere + x, 0)
        maxsofar = max(maxsofar, maxendinghere)
    return maxsofar


def maxsumseq(sequence):
    start, end, sum_start = -1, -1, -1
    maxsum_, sum_ = 0, 0
    for i, x in enumerate(sequence):
        sum_ += x
        if maxsum_ < sum_: # found maximal subsequence so far
            maxsum_ = sum_
            start, end = sum_start, i
        elif sum_ < 0: # start new sequence
            sum_ = 0
            sum_start = i
    assert maxsum_ == maxsum(sequence)
    assert maxsum_ == sum(sequence[start + 1:end + 1])
    return sequence[start + 1:end + 1]


def maxsumit(iterable):
    maxseq = seq = []
    start, end, sum_start = -1, -1, -1
    maxsum_, sum_ = 0, 0
    for i, x in enumerate(iterable):
        seq.append(x); sum_ += x
        if maxsum_ < sum_:
            maxseq = seq; maxsum_ = sum_
            start, end = sum_start, i
        elif sum_ < 0:
            seq = []; sum_ = 0
            sum_start = i
    assert maxsum_ == sum(maxseq[:end - start])
    return maxseq[:end - start]



from pythran.tests import TestEnv
from pythran.typing import List, Set, Dict, NDArray
import numpy as np

class TestNoGil(TestEnv):

    def test_list_param(self):
        code="""
        def list_param(l):
            return l, sum(i*j for i in l for j in l)
        """
        self.run_test(code, list(range(30)), list_param=[List[int]],
                      thread_count=4)

    def test_set_param(self):
        code="""
        def set_param(l):
            return {sum(l), sum(i*j for i in l for j in l)}, l
        """
        self.run_test(code, set(range(30)), set_param=[Set[int]],
                      thread_count=4)

    def test_dict_param(self):
        code="""
        def dict_param(l):
            return {sum(i-j for i in l.keys() for j in l.values()): l}, l
        """
        self.run_test(code, dict(zip(range(30), range(30))),
                      dict_param=[Dict[int, int]],
                      thread_count=4)

    def test_ndarray_param(self):
        code="""
        import numpy as np
        def ndarray_param(l):
            return np.array([i*j for i in l for j in l]), l
        """
        self.run_test(code, np.ones(100, dtype=int),
                      ndarray_param=[NDArray[int, :]],
                      thread_count=4)




#runas solve()
#unittest.skip recursive generator
#pythran export solve()
def solve():
    '''
    In England the currency is made up of pound, P, and pence, p, and there are eight coins in general circulation:

    1p, 2p, 5p, 10p, 20p, 50p, P1 (100p) and P2 (200p).
    It is possible to make P2 in the following way:

    1 P1 + 1 50p + 2 20p + 1 5p + 1 2p + 3 1p
    How many different ways can P2 be made using any number of coins?
    '''

    coins = [1, 2, 5, 10, 20, 50, 100, 200]

    def balance(pattern): return sum(coins[x]*pattern[x] for x in range(0, len(pattern)))

    def gen(pattern, coinnum, num):
        coin = coins[coinnum]
        for p in range(0, num/coin + 1):
            newpat = pattern[:coinnum] + (p,)
            bal = balance(newpat)
            if bal > num: return
            elif bal == num: yield newpat
            elif coinnum < len(coins)-1:
                for pat in gen(newpat, coinnum+1, num):
                    yield pat

    return sum(1 for pat in gen((), 0, 200))





def omp_parallel_for_lastprivate():
    sum = 0
    i0 = -1
    'omp parallel for reduction(+:sum) schedule(static,7) lastprivate(i0)'
    for i in range(1,1001):
        sum += i
        i0 = i

    known_sum = (1000 * (1000 + 1)) / 2
    return known_sum == sum and i0 == 1000



#from https://github.com/serge-sans-paille/pythran/issues/1229
#runas import numpy as np; x = np.arange(3., 10.); empirical(x, 3., .5)
import numpy as np

#pythran export empirical(float[:], float, float)
def empirical(ds, alpha, x):
    sds = np.sort(ds)
    ds_to_the_alpha = sds**alpha
    fractions = ds_to_the_alpha #/ sum (ds_to_the_alpha)
    thresholds = np.cumsum(fractions)
    thresholds /= thresholds[-1]
    i = find_first (thresholds, lambda u: x < u)
    return i

#pthran export find_first(float[:], bool (float))
def find_first (seq, pred):
    for i,x in enumerate (seq):
        print(i, x, pred(x))
        if pred(x):
            return i
    return None





#from http://rosettacode.org/wiki/100_doors#Python
#pythran export unoptimized()
#runas unoptimized()
#pythran export optimized()
#runas optimized()
#pythran export one_liner_list_comprehension()
#runas one_liner_list_comprehension()
#pythran export one_liner_generator_comprehension()
#runas one_liner_generator_comprehension()

def unoptimized():
    doors = [False] * 100
    for i in range(100):
        for j in range(i, 100, i+1):
            doors[j] = not doors[j]
        print("Door %d:" % (i+1), 'open' if doors[i] else 'close')

def optimized():
    for i in range(1, 101):
        root = i ** 0.5
        print("Door %d:" % i, 'open' if root == int(root) else 'close')

def one_liner_list_comprehension():
    print('\n'.join(['Door %s is %s' % (i, ['closed', 'open'][(i**0.5).is_integer()]) for i in range(1, 10001)]))

def one_liner_generator_comprehension():
    print('\n'.join('Door %s is %s' % (i, 'closed' if i**0.5 % 1 else 'open') for i in range(1, 101)))



#pythran export average_position(str:(float,float) dict, str:(float,float) dict)
#runas d = {"e":(1.,2.) } ; e = {"d":(2.,1.) } ; average_position(e,d)

def average_position(pos1,pos2):
	pos_avg={}
	for k in pos1:
		if k in pos2:
			pos_avg[k]=((pos1[k][0]+pos2[k][0])/2,(pos1[k][1]+pos2[k][1])/2)
		else:
			pos_avg[k]=pos1[k]
	for k in pos2:
		if k in pos1:
			if k not in pos_avg:
				pos_avg[k]=((pos1[k][0]+pos2[k][0])/2,(pos1[k][1]+pos2[k][1])/2)
		else:
			pos_avg[k]=pos2[k]
	return pos_avg
